from enum import Enum as Enum, IntEnum, StrEnum

class Ac3DPolylineType(IntEnum):
    acCubicSpline3DPoly = 2
    acQuadSpline3DPoly = 1
    acSimple3DPoly = 0

class AcARXDemandLoad(IntEnum):
    acDemanLoadDisable = 0
    acDemandLoadCmdInvoke = 2
    acDemandLoadOnObjectDetect = 1

class AcActiveSpace(IntEnum):
    acModelSpace = 1
    acPaperSpace = 0

class AcAlignment(IntEnum):
    acAlignmentAligned = 3
    acAlignmentBottomCenter = 13
    acAlignmentBottomLeft = 12
    acAlignmentBottomRight = 14
    acAlignmentCenter = 1
    acAlignmentFit = 5
    acAlignmentLeft = 0
    acAlignmentMiddle = 4
    acAlignmentMiddleCenter = 10
    acAlignmentMiddleLeft = 9
    acAlignmentMiddleRight = 11
    acAlignmentRight = 2
    acAlignmentTopCenter = 7
    acAlignmentTopLeft = 6
    acAlignmentTopRight = 8

class AcAlignmentPointAcquisition(IntEnum):
    acAlignPntAcquisitionAutomatic = 0
    acAlignPntAcquisitionShiftToAcquire = 1

class AcAngleUnits(IntEnum):
    acDegreeMinuteSeconds = 1
    acDegrees = 0
    acGrads = 2
    acRadians = 3

class AcAttachmentPoint(IntEnum):
    acAttachmentPointBottomCenter = 8
    acAttachmentPointBottomLeft = 7
    acAttachmentPointBottomRight = 9
    acAttachmentPointMiddleCenter = 5
    acAttachmentPointMiddleLeft = 4
    acAttachmentPointMiddleRight = 6
    acAttachmentPointTopCenter = 2
    acAttachmentPointTopLeft = 1
    acAttachmentPointTopRight = 3

class AcAttributeMode(IntEnum):
    acAttributeModeConstant = 2
    acAttributeModeInvisible = 1
    acAttributeModeLockPosition = 16
    acAttributeModeMultipleLine = 32
    acAttributeModeNormal = 0
    acAttributeModePreset = 8
    acAttributeModeVerify = 4

class AcBlockConnectionType(IntEnum):
    acConnectBase = 1
    acConnectExtents = 0

class AcBlockScaling(IntEnum):
    acAny = 0
    acUniform = 1

class AcBoolean(IntEnum):
    acFalse = 0
    acTrue = 1

class AcBooleanType(IntEnum):
    acIntersection = 1
    acSubtraction = 2
    acUnion = 0

class AcCellAlignment(IntEnum):
    acBottomCenter = 8
    acBottomLeft = 7
    acBottomRight = 9
    acMiddleCenter = 5
    acMiddleLeft = 4
    acMiddleRight = 6
    acTopCenter = 2
    acTopLeft = 1
    acTopRight = 3

class AcCellContentLayout(IntEnum):
    acCellContentLayoutFlow = 1
    acCellContentLayoutStackedHorizontal = 2
    acCellContentLayoutStackedVertical = 4

class AcCellContentType(IntEnum):
    acCellContentTypeBlock = 4
    acCellContentTypeField = 2
    acCellContentTypeUnknown = 0
    acCellContentTypeValue = 1

class AcCellEdgeMask(IntEnum):
    acBottomMask = 4
    acLeftMask = 8
    acRightMask = 2
    acTopMask = 1

class AcCellMargin(IntEnum):
    acCellMarginBottom = 4
    acCellMarginHorzSpacing = 16
    acCellMarginLeft = 2
    acCellMarginRight = 8
    acCellMarginTop = 1
    acCellMarginVertSpacing = 32

class AcCellOption(IntEnum):
    kCellOptionNone = 0
    kInheritCellFormat = 1

class AcCellProperty(IntEnum):
    acAlignmentProperty = 32
    acAllCellProperties = 524287
    acAutoScale = 32768
    acBackgroundColor = 128
    acBitProperties = 245760
    acContentColor = 64
    acContentLayout = 262144
    acContentProperties = 33662
    acDataFormat = 4
    acDataType = 2
    acDataTypeAndFormat = 6
    acEnableBackgroundColor = 16384
    acFlowDirBtoT = 131072
    acInvalidCellProperty = 0
    acLock = 1
    acMarginBottom = 8192
    acMarginLeft = 1024
    acMarginRight = 4096
    acMarginTop = 2048
    acMergeAll = 65536
    acRotation = 8
    acScale = 16
    acTextHeight = 512
    acTextStyle = 256

class AcCellState(IntEnum):
    acCellStateContentLocked = 1
    acCellStateContentModified = 32
    acCellStateContentReadOnly = 2
    acCellStateFormatLocked = 4
    acCellStateFormatModified = 64
    acCellStateFormatReadOnly = 8
    acCellStateLinked = 16
    acCellStateNone = 0

class AcCellType(IntEnum):
    acBlockCell = 2
    acTextCell = 1
    acUnknownCell = 0

class AcColor(IntEnum):
    acByBlock = 0
    acByLayer = 256
    acRed = 1
    acYellow = 2
    acGreen = 3
    acCyan = 4
    acBlue = 5
    acMagenta = 6
    acWhite = 7
    attr_name = ...

class AcColorMethod(IntEnum):
    acColorMethodByACI = 195
    acColorMethodByBlock = 193
    acColorMethodByLayer = 192
    acColorMethodByRGB = 194
    acColorMethodForeground = 197

class AcCoordinateSystem(IntEnum):
    acDisplayDCS = 2
    acOCS = 4
    acPaperSpaceDCS = 3
    acUCS = 1
    acWorld = 0

class AcDataLinkUpdateDirection(IntEnum):
    acUpdateDataFromSource = 1
    acUpdateSourceFromData = 2

class AcDataLinkUpdateOption(IntEnum):
    acUpdateOptionIncludeXrefs = 1048576
    acUpdateOptionNone = 0
    acUpdateOptionOverwriteContentModifiedAfterUpdate = 131072
    acUpdateOptionOverwriteFormatModifiedAfterUpdate = 262144
    acUpdateOptionUpdateFullSourceRange = 524288

class AcDimArcLengthSymbol(IntEnum):
    acSymAbove = 1
    acSymInFront = 0
    acSymNone = 2

class AcDimArrowheadType(IntEnum):
    acArrowArchTick = 4
    acArrowBoxBlank = 14
    acArrowBoxFilled = 15
    acArrowClosed = 2
    acArrowClosedBlank = 1
    acArrowDatumBlank = 16
    acArrowDatumFilled = 17
    acArrowDefault = 0
    acArrowDot = 3
    acArrowDotBlank = 12
    acArrowDotSmall = 11
    acArrowIntegral = 18
    acArrowNone = 19
    acArrowOblique = 5
    acArrowOpen = 6
    acArrowOpen30 = 10
    acArrowOpen90 = 9
    acArrowOrigin = 7
    acArrowOrigin2 = 8
    acArrowSmall = 13
    acArrowUserDefined = 20

class AcDimCenterType(IntEnum):
    acCenterLine = 1
    acCenterMark = 0
    acCenterNone = 2

class AcDimFit(IntEnum):
    acArrowsOnly = 1
    acBestFit = 3
    acTextAndArrows = 0
    acTextOnly = 2

class AcDimFractionType(IntEnum):
    acDiagonal = 1
    acHorizontal = 0
    acNotStacked = 2

class AcDimHorizontalJustification(IntEnum):
    acFirstExtensionLine = 1
    acHorzCentered = 0
    acOverFirstExtension = 3
    acOverSecondExtension = 4
    acSecondExtensionLine = 2

class AcDimLUnits(IntEnum):
    acDimLArchitectural = 4
    acDimLDecimal = 2
    acDimLEngineering = 3
    acDimLFractional = 5
    acDimLScientific = 1
    acDimLWindowsDesktop = 6

class AcDimPrecision(IntEnum):
    acDimPrecisionEight = 8
    acDimPrecisionFive = 5
    acDimPrecisionFour = 4
    acDimPrecisionOne = 1
    acDimPrecisionSeven = 7
    acDimPrecisionSix = 6
    acDimPrecisionThree = 3
    acDimPrecisionTwo = 2
    acDimPrecisionZero = 0

class AcDimTextMovement(IntEnum):
    acDimLineWithText = 0
    acMoveTextAddLeader = 1
    acMoveTextNoLeader = 2

class AcDimToleranceJustify(IntEnum):
    acTolBottom = 0
    acTolMiddle = 1
    acTolTop = 2

class AcDimToleranceMethod(IntEnum):
    acTolBasic = 4
    acTolDeviation = 2
    acTolLimits = 3
    acTolNone = 0
    acTolSymmetrical = 1

class AcDimUnits(IntEnum):
    acDimArchitectural = 6
    acDimArchitecturalStacked = 4
    acDimDecimal = 2
    acDimEngineering = 3
    acDimFractional = 7
    acDimFractionalStacked = 5
    acDimScientific = 1
    acDimWindowsDesktop = 8

class AcDimVerticalJustification(IntEnum):
    acAbove = 1
    acJIS = 3
    acOutside = 2
    acUnder = 4
    acVertCentered = 0

class AcDragDisplayMode(IntEnum):
    acDragDisplayAutomatically = 2
    acDragDisplayOnRequest = 1
    acDragDoNotDisplay = 0

class AcDrawLeaderOrderType(IntEnum):
    acDrawLeaderHeadFirst = 0
    acDrawLeaderTailFirst = 1

class AcDrawMLeaderOrderType(IntEnum):
    acDrawContentFirst = 0
    acDrawLeaderFirst = 1

class AcDrawingAreaSCMCommand(IntEnum):
    acEnableSCM = 2
    acEnableSCMOptions = 1
    acEnter = 0

class AcDrawingAreaSCMDefault(IntEnum):
    acRepeatLastCommand = 0
    acSCM = 1

class AcDrawingAreaSCMEdit(IntEnum):
    acEdRepeatLastCommand = 0
    acEdSCM = 1

class AcDrawingAreaShortCutMenu(IntEnum):
    acNoDrawingAreaShortCutMenu = 0
    acUseDefaultDrawingAreaShortCutMenu = 1

class AcDrawingDirection(IntEnum):
    acBottomToTop = 4
    acByStyle = 5
    acLeftToRight = 1
    acRightToLeft = 2
    acTopToBottom = 3

class AcDynamicBlockReferencePropertyUnitsType(IntEnum):
    acAngular = 1
    acArea = 3
    acDistance = 2
    acNoUnits = 0

class AcEntityName(IntEnum):
    ac3dFace = 1
    ac3dPolyline = 2
    ac3dSolid = 3
    acArc = 4
    acAttribute = 5
    acAttributeReference = 6
    acBlockReference = 7
    acCircle = 8
    acDgnUnderlay = 47
    acDim3PointAngular = 41
    acDimAligned = 9
    acDimAngular = 10
    acDimArcLength = 44
    acDimDiametric = 12
    acDimOrdinate = 13
    acDimRadial = 14
    acDimRadialLarge = 45
    acDimRotated = 15
    acDwfUnderlay = 46
    acEllipse = 16
    acExternalReference = 42
    acGroup = 37
    acHatch = 17
    acLeader = 18
    acLine = 19
    acMInsertBlock = 38
    acMLeader = 48
    acMLine = 40
    acMtext = 21
    acNurbSurface = 51
    acPViewport = 35
    acPdfUnderlay = 50
    acPoint = 22
    acPolyfaceMesh = 39
    acPolyline = 23
    acPolylineLight = 24
    acPolymesh = 25
    acRaster = 26
    acRay = 27
    acRegion = 28
    acShape = 29
    acSolid = 30
    acSpline = 31
    acSubDMesh = 49
    acTable = 43
    acText = 32
    acTolerance = 33
    acTrace = 34
    acXline = 36

class AcExtendOption(IntEnum):
    acExtendBoth = 3
    acExtendNone = 0
    acExtendOtherEntity = 2
    acExtendThisEntity = 1

class AcFormatOption(IntEnum):
    acForEditing = 1
    acForExpression = 2
    acIgnoreMtextFormat = 8
    acUseMaximumPrecision = 4
    kFormatOptionNone = 0

class AcGradientPatternType(IntEnum):
    acPreDefinedGradient = 0
    acUserDefinedGradient = 1

class AcGridLineStyle(IntEnum):
    acGridLineStyleDouble = 2
    acGridLineStyleSingle = 1

class AcGridLineType(IntEnum):
    acHorzBottom = 4
    acHorzInside = 2
    acHorzTop = 1
    acInvalidGridLine = 0
    acVertInside = 16
    acVertLeft = 8
    acVertRight = 32

class AcHatchObjectType(IntEnum):
    acGradientObject = 1
    acHatchObject = 0

class AcHatchStyle(IntEnum):
    acHatchStyleIgnore = 2
    acHatchStyleNormal = 0
    acHatchStyleOuter = 1

class AcHelixConstrainType(IntEnum):
    acHeight = 2
    acTurnHeight = 0
    acTurns = 1

class AcHelixTwistType(IntEnum):
    acCCW = 0
    acCW = 1

class AcHorizontalAlignment(IntEnum):
    acHorizontalAlignmentAligned = 3
    acHorizontalAlignmentCenter = 1
    acHorizontalAlignmentFit = 5
    acHorizontalAlignmentLeft = 0
    acHorizontalAlignmentMiddle = 4
    acHorizontalAlignmentRight = 2

class AcISOPenWidth(IntEnum):
    acPenWidth013 = 13
    acPenWidth018 = 18
    acPenWidth025 = 25
    acPenWidth035 = 35
    acPenWidth050 = 50
    acPenWidth070 = 70
    acPenWidth100 = 100
    acPenWidth140 = 140
    acPenWidth200 = 200
    acPenWidthUnk = -1

class AcInsertUnits(IntEnum):
    acInsertUnitsAngstroms = 11
    acInsertUnitsAstronomicalUnits = 18
    acInsertUnitsCentimeters = 5
    acInsertUnitsDecameters = 15
    acInsertUnitsDecimeters = 14
    acInsertUnitsFeet = 2
    acInsertUnitsGigameters = 17
    acInsertUnitsHectometers = 16
    acInsertUnitsInches = 1
    acInsertUnitsKilometers = 7
    acInsertUnitsLightYears = 19
    acInsertUnitsMeters = 6
    acInsertUnitsMicroinches = 8
    acInsertUnitsMicrons = 13
    acInsertUnitsMiles = 3
    acInsertUnitsMillimeters = 4
    acInsertUnitsMils = 9
    acInsertUnitsNanometers = 12
    acInsertUnitsParsecs = 20
    acInsertUnitsUSSurveyFeet = 21
    acInsertUnitsUSSurveyInch = 22
    acInsertUnitsUSSurveyMile = 24
    acInsertUnitsUSSurveyYard = 23
    acInsertUnitsUnitless = 0
    acInsertUnitsYards = 10

class AcInsertUnitsAction(IntEnum):
    acInsertUnitsAutoAssign = 1
    acInsertUnitsPrompt = 0

class AcKeyboardAccelerator(IntEnum):
    acPreferenceClassic = 0
    acPreferenceCustom = 1

class AcKeyboardPriority(IntEnum):
    acKeyboardEntry = 1
    acKeyboardEntryExceptScripts = 2
    acKeyboardRunningObjSnap = 0

class AcLayerStateMask(IntEnum):
    acLsAll = 65535
    acLsColor = 32
    acLsFrozen = 2
    acLsLineType = 64
    acLsLineWeight = 128
    acLsLocked = 4
    acLsNewViewport = 16
    acLsNone = 0
    acLsOn = 1
    acLsPlot = 8
    acLsPlotStyle = 256

class AcLeaderType(IntEnum):
    acLineNoArrow = 0
    acLineWithArrow = 2
    acSplineNoArrow = 1
    acSplineWithArrow = 3

class AcLineSpacingStyle(IntEnum):
    acLineSpacingStyleAtLeast = 1
    acLineSpacingStyleExactly = 2

class AcLineWeight(IntEnum):
    acLnWt000 = 0
    acLnWt005 = 5
    acLnWt009 = 9
    acLnWt013 = 13
    acLnWt015 = 15
    acLnWt018 = 18
    acLnWt020 = 20
    acLnWt025 = 25
    acLnWt030 = 30
    acLnWt035 = 35
    acLnWt040 = 40
    acLnWt050 = 50
    acLnWt053 = 53
    acLnWt060 = 60
    acLnWt070 = 70
    acLnWt080 = 80
    acLnWt090 = 90
    acLnWt100 = 100
    acLnWt106 = 106
    acLnWt120 = 120
    acLnWt140 = 140
    acLnWt158 = 158
    acLnWt200 = 200
    acLnWt211 = 211
    acLnWtByBlock = -2
    acLnWtByLayer = -1
    acLnWtByLwDefault = -3

class AcLoadPalette(IntEnum):
    acPaletteByDrawing = 0
    acPaletteBySession = 1

class AcLoftedSurfaceNormalType(IntEnum):
    acAllNormal = 5
    acEndsNormal = 4
    acFirstNormal = 2
    acLastNormal = 3
    acRuled = 0
    acSmooth = 1
    acUseDraftAngles = 6

class AcLoopType(IntEnum):
    acHatchLoopTypeDefault = 0
    acHatchLoopTypeDerived = 4
    acHatchLoopTypeExternal = 1
    acHatchLoopTypePolyline = 2
    acHatchLoopTypeTextbox = 8

class AcMLeaderContentType(IntEnum):
    acBlockContent = 1
    acMTextContent = 2
    acNoneContent = 0

class AcMLeaderType(IntEnum):
    acInVisibleLeader = 0
    acSplineLeader = 2
    acStraightLeader = 1

class AcMLineJustification(IntEnum):
    acBottom = 2
    acTop = 0
    acZero = 1

class AcMeasurementUnits(IntEnum):
    acEnglish = 0
    acMetric = 1

class AcMenuFileType(IntEnum):
    acMenuFileCompiled = 0
    acMenuFileSource = 1

class AcMenuGroupType(IntEnum):
    acBaseMenuGroup = 0
    acPartialMenuGroup = 1

class AcMenuItemType(IntEnum):
    acMenuItem = 0
    acMenuSeparator = 1
    acMenuSubMenu = 2

class AcMergeCellStyleOption(IntEnum):
    acMergeCellStyleConvertDuplicatesToOverrides = 4
    acMergeCellStyleCopyDuplicates = 1
    acMergeCellStyleIgnoreNewStyles = 8
    acMergeCellStyleNone = 0
    acMergeCellStyleOverwriteDuplicates = 2

class AcMeshCreaseType(IntEnum):
    acAlwaysCrease = 1
    acCreaseByLevel = 2
    acNoneCrease = 0

class AcOlePlotQuality(IntEnum):
    acOPQHighGraphics = 2
    acOPQLowGraphics = 1
    acOPQMonochrome = 0

class AcOleQuality(IntEnum):
    acOQGraphics = 2
    acOQHighPhoto = 4
    acOQLineArt = 0
    acOQPhoto = 3
    acOQText = 1

class AcOleType(IntEnum):
    acOTEmbedded = 2
    acOTLink = 1
    acOTStatic = 3

class AcOnOff(IntEnum):
    acOff = 0
    acOn = 1

class AcParseOption(IntEnum):
    acParseOptionNone = 0
    acPreserveMtextFormat = 2
    acSetDefaultFormat = 1

class AcPatternType(IntEnum):
    acHatchPatternTypeCustomDefined = 2
    acHatchPatternTypePreDefined = 1
    acHatchPatternTypeUserDefined = 0

class AcPlotOrientation(IntEnum):
    acPlotOrientationLandscape = 1
    acPlotOrientationPortrait = 0

class AcPlotPaperUnits(IntEnum):
    acInches = 0
    acMillimeters = 1
    acPixels = 2

class AcPlotPolicy(IntEnum):
    acPolicyLegacy = 1
    acPolicyNamed = 0

class AcPlotPolicyForLegacyDwgs(IntEnum):
    acPolicyLegacyDefault = 0
    acPolicyLegacyLegacy = 2
    acPolicyLegacyQuery = 1

class AcPlotPolicyForNewDwgs(IntEnum):
    acPolicyNewDefault = 0
    acPolicyNewLegacy = 1

class AcPlotRotation(IntEnum):
    ac0degrees = 0
    ac180degrees = 2
    ac270degrees = 3
    ac90degrees = 1

class AcPlotScale(IntEnum):
    ac100_1 = 32
    ac10_1 = 31
    ac1_1 = 16
    ac1_10 = 21
    ac1_100 = 27
    ac1_128in_1ft = 1
    ac1_16 = 22
    ac1_16in_1ft = 4
    ac1_2 = 17
    ac1_20 = 23
    ac1_2in_1ft = 10
    ac1_30 = 24
    ac1_32in_1ft = 3
    ac1_4 = 18
    ac1_40 = 25
    ac1_4in_1ft = 8
    ac1_5 = 19
    ac1_50 = 26
    ac1_64in_1ft = 2
    ac1_8 = 20
    ac1_8in_1ft = 6
    ac1ft_1ft = 15
    ac1in_1ft = 12
    ac2_1 = 28
    ac3_16in_1ft = 7
    ac3_32in_1ft = 5
    ac3_4in_1ft = 11
    ac3_8in_1ft = 9
    ac3in_1ft = 13
    ac4_1 = 29
    ac6in_1ft = 14
    ac8_1 = 30
    acScaleToFit = 0

class AcPlotType(IntEnum):
    acDisplay = 0
    acExtents = 1
    acLayout = 5
    acLimits = 2
    acView = 3
    acWindow = 4

class AcPointCloudColorType(IntEnum):
    acByColor = 1
    acTrueColor = 0

class AcPointCloudExStylizationType(IntEnum):
    acClassification = 5
    acElevation = 4
    acIntensities = 3
    acNormals = 2
    acObject = 1
    acRGB = 0

class AcPointCloudIntensityStyle(IntEnum):
    acIntensityBlue = 4
    acIntensityEditableFlag = 5
    acIntensityGrayscale = 0
    acIntensityGreen = 3
    acIntensityRainbow = 1
    acIntensityRed = 2

class AcPointCloudStylizationType(IntEnum):
    acIntensity = 3
    acNormal = 2
    acObjectColor = 1
    acScanColor = 0

class AcPolylineType(IntEnum):
    acCubicSplinePoly = 3
    acFitCurvePoly = 1
    acQuadSplinePoly = 2
    acSimplePoly = 0

class AcPolymeshType(IntEnum):
    acBezierSurfaceMesh = 8
    acCubicSurfaceMesh = 6
    acQuadSurfaceMesh = 5
    acSimpleMesh = 0

class AcPredefBlockType(IntEnum):
    acBlockBox = 3
    acBlockCircle = 2
    acBlockHexagon = 4
    acBlockImperial = 0
    acBlockSlot = 1
    acBlockTriangle = 5
    acBlockUserDefined = 6

class AcPreviewMode(IntEnum):
    acFullPreview = 1
    acPartialPreview = 0

class AcPrinterSpoolAlert(IntEnum):
    acPrinterAlertOnce = 1
    acPrinterAlwaysAlert = 0
    acPrinterNeverAlert = 3
    acPrinterNeverAlertLogOnce = 2

class AcProxyImage(IntEnum):
    acProxyBoundingBox = 2
    acProxyNotShow = 0
    acProxyShow = 1

class AcRegenType(IntEnum):
    acActiveViewport = 0
    acAllViewports = 1

class AcRotationAngle(IntEnum):
    acDegrees000 = 0
    acDegrees090 = 1
    acDegrees180 = 2
    acDegrees270 = 3
    acDegreesUnknown = -1

class AcRowType(IntEnum):
    acDataRow = 1
    acHeaderRow = 4
    acTitleRow = 2
    acUnknownRow = 0

class AcSaveAsType(IntEnum):
    ac2000_Template = 14
    ac2000_dwg = 12
    ac2000_dxf = 13
    ac2004_Template = 26
    ac2004_dwg = 24
    ac2004_dxf = 25
    ac2007_Template = 38
    ac2007_dwg = 36
    ac2007_dxf = 37
    ac2010_Template = 50
    ac2010_dwg = 48
    ac2010_dxf = 49
    ac2013_Template = 62
    ac2013_dwg = 60
    ac2013_dxf = 61
    ac2018_Template = 66
    ac2018_dwg = 64
    ac2018_dxf = 65
    acNative = 64
    acR12_dxf = 1
    acR13_dwg = 4
    acR13_dxf = 5
    acR14_dwg = 8
    acR14_dxf = 9
    acR15_Template = 14
    acR15_dwg = 12
    acR15_dxf = 13
    acR18_Template = 26
    acR18_dwg = 24
    acR18_dxf = 25
    acUnknown = -1

class AcSectionGeneration(IntEnum):
    acSectionGenerationDestinationFile = 64
    acSectionGenerationDestinationNewBlock = 16
    acSectionGenerationDestinationReplaceBlock = 32
    acSectionGenerationSourceAllObjects = 1
    acSectionGenerationSourceSelectedObjects = 2

class AcSectionState(IntEnum):
    acSectionStateBoundary = 2
    acSectionStatePlane = 1
    acSectionStateVolume = 4

class AcSectionState2(IntEnum):
    acSectionState2Boundary = 4
    acSectionState2Plane = 1
    acSectionState2Slice = 2
    acSectionState2Volume = 8

class AcSectionSubItem(IntEnum):
    acSectionSubItemBackLine = 8
    acSectionSubItemBackLineBottom = 32
    acSectionSubItemBackLineTop = 16
    acSectionSubItemSectionLine = 1
    acSectionSubItemSectionLineBottom = 4
    acSectionSubItemSectionLineTop = 2
    acSectionSubItemVerticalLineBottom = 128
    acSectionSubItemVerticalLineTop = 64
    acSectionSubItemkNone = 0

class AcSectionType(IntEnum):
    acSectionType2dSection = 2
    acSectionType3dSection = 4
    acSectionTypeLiveSection = 1

class AcSegmentAngleType(IntEnum):
    acDegrees15 = 1
    acDegrees30 = 2
    acDegrees45 = 3
    acDegrees60 = 4
    acDegrees90 = 6
    acDegreesAny = 0
    acDegreesHorz = 12

class AcSelect(IntEnum):
    acSelectionSetAll = 5
    acSelectionSetCrossing = 1
    acSelectionSetCrossingPolygon = 7
    acSelectionSetFence = 2
    acSelectionSetLast = 4
    acSelectionSetPrevious = 3
    acSelectionSetWindow = 0
    acSelectionSetWindowPolygon = 6

class AcSelectType(IntEnum):
    acTableSelectCrossing = 2
    acTableSelectWindow = 1

class AcShadePlot(IntEnum):
    acShadePlotAsDisplayed = 0
    acShadePlotHidden = 2
    acShadePlotRendered = 3
    acShadePlotWireframe = 1

class AcShadowDisplayType(IntEnum):
    acCastsAndReceivesShadows = 0
    acCastsShadows = 1
    acIgnoreShadows = 3
    acReceivesShadows = 2

class AcSplineFrameType(IntEnum):
    acHide = 1
    acShow = 0

class AcSplineKnotParameterizationType(IntEnum):
    acChord = 0
    acCustomParameterization = 15
    acSqrtChord = 1
    acUniformParam = 2

class AcSplineMethodType(IntEnum):
    acControlVertices = 1
    acFit = 0

class AcTableDirection(IntEnum):
    acTableBottomToTop = 1
    acTableTopToBottom = 0

class AcTableFlowDirection(IntEnum):
    acTableFlowDownOrUp = 2
    acTableFlowLeft = 4
    acTableFlowRight = 1

class AcTableStyleOverrides(IntEnum):
    acCellAlign = 130
    acCellBackgroundColor = 132
    acCellBackgroundFillNone = 131
    acCellBottomGridColor = 138
    acCellBottomGridLineWeight = 142
    acCellBottomVisibility = 146
    acCellContentColor = 133
    acCellDataType = 148
    acCellLeftGridColor = 139
    acCellLeftGridLineWeight = 143
    acCellLeftVisibility = 147
    acCellRightGridColor = 137
    acCellRightGridLineWeight = 141
    acCellRightVisibility = 145
    acCellTextHeight = 135
    acCellTextStyle = 134
    acCellTopGridColor = 136
    acCellTopGridLineWeight = 140
    acCellTopVisibility = 144
    acDataHorzBottomColor = 54
    acDataHorzBottomLineWeight = 84
    acDataHorzBottomVisibility = 114
    acDataHorzInsideColor = 53
    acDataHorzInsideLineWeight = 83
    acDataHorzInsideVisibility = 113
    acDataHorzTopColor = 52
    acDataHorzTopLineWeight = 82
    acDataHorzTopVisibility = 112
    acDataRowAlignment = 17
    acDataRowColor = 8
    acDataRowDataType = 26
    acDataRowFillColor = 14
    acDataRowFillNone = 11
    acDataRowTextHeight = 23
    acDataRowTextStyle = 20
    acDataVertInsideColor = 56
    acDataVertInsideLineWeight = 86
    acDataVertInsideVisibility = 116
    acDataVertLeftColor = 55
    acDataVertLeftLineWeight = 85
    acDataVertLeftVisibility = 115
    acDataVertRightColor = 57
    acDataVertRightLineWeight = 87
    acDataVertRightVisibility = 117
    acFlowDirection = 3
    acHeaderHorzBottomColor = 48
    acHeaderHorzBottomLineWeight = 78
    acHeaderHorzBottomVisibility = 108
    acHeaderHorzInsideColor = 47
    acHeaderHorzInsideLineWeight = 77
    acHeaderHorzInsideVisibility = 107
    acHeaderHorzTopColor = 46
    acHeaderHorzTopLineWeight = 76
    acHeaderHorzTopVisibility = 106
    acHeaderRowAlignment = 16
    acHeaderRowColor = 7
    acHeaderRowDataType = 25
    acHeaderRowFillColor = 13
    acHeaderRowFillNone = 10
    acHeaderRowTextHeight = 22
    acHeaderRowTextStyle = 19
    acHeaderSuppressed = 2
    acHeaderVertInsideColor = 50
    acHeaderVertInsideLineWeight = 80
    acHeaderVertInsideVisibility = 110
    acHeaderVertLeftColor = 49
    acHeaderVertLeftLineWeight = 79
    acHeaderVertLeftVisibility = 109
    acHeaderVertRightColor = 51
    acHeaderVertRightLineWeight = 81
    acHeaderVertRightVisibility = 111
    acHorzCellMargin = 4
    acTitleHorzBottomColor = 42
    acTitleHorzBottomLineWeight = 72
    acTitleHorzBottomVisibility = 102
    acTitleHorzInsideColor = 41
    acTitleHorzInsideLineWeight = 71
    acTitleHorzInsideVisibility = 101
    acTitleHorzTopColor = 40
    acTitleHorzTopLineWeight = 70
    acTitleHorzTopVisibility = 100
    acTitleRowAlignment = 15
    acTitleRowColor = 6
    acTitleRowDataType = 24
    acTitleRowFillColor = 12
    acTitleRowFillNone = 9
    acTitleRowTextHeight = 21
    acTitleRowTextStyle = 18
    acTitleSuppressed = 1
    acTitleVertInsideColor = 44
    acTitleVertInsideLineWeight = 74
    acTitleVertInsideVisibility = 104
    acTitleVertLeftColor = 43
    acTitleVertLeftLineWeight = 73
    acTitleVertLeftVisibility = 103
    acTitleVertRightColor = 45
    acTitleVertRightLineWeight = 75
    acTitleVertRightVisibility = 105
    acVertCellMargin = 5

class AcTextAlignmentType(IntEnum):
    acCenterAlignment = 1
    acLeftAlignment = 0
    acRightAlignment = 2

class AcTextAngleType(IntEnum):
    acAlwaysRightReadingAngle = 2
    acHorizontalAngle = 1
    acInsertAngle = 0

class AcTextAttachmentDirection(IntEnum):
    acAttachmentHorizontal = 0
    acAttachmentVertical = 1

class AcTextAttachmentType(IntEnum):
    acAttachmentAllLine = 8
    acAttachmentBottomLine = 7
    acAttachmentBottomOfBottom = 6
    acAttachmentBottomOfTop = 2
    acAttachmentBottomOfTopLine = 3
    acAttachmentMiddle = 4
    acAttachmentMiddleOfBottom = 5
    acAttachmentMiddleOfTop = 1
    acAttachmentTopOfTop = 0

class AcTextFontStyle(IntEnum):
    acFontBold = 2
    acFontBoldItalic = 3
    acFontItalic = 1
    acFontRegular = 0

class AcTextGenerationFlag(IntEnum):
    acTextNoFlag = 0
    acTextFlagBackward = 2
    acTextFlagUpsideDown = 4

class AcToolbarDockStatus(IntEnum):
    acToolbarDockBottom = 1
    acToolbarDockLeft = 2
    acToolbarDockRight = 3
    acToolbarDockTop = 0
    acToolbarFloating = 4

class AcToolbarItemType(IntEnum):
    acToolbarButton = 0
    acToolbarControl = 2
    acToolbarFlyout = 3
    acToolbarSeparator = 1

class AcUnderlayLayerOverrideType(IntEnum):
    acApplied = 1
    acNoOverrides = 0

class AcUnits(IntEnum):
    acArchitectural = 4
    acDecimal = 2
    acDefaultUnits = -1
    acEngineering = 3
    acFractional = 5
    acScientific = 1

class AcValueDataType(IntEnum):
    acBuffer = 128
    acDate = 8
    acDouble = 2
    acGeneral = 512
    acLong = 1
    acObjectId = 64
    acPoint2d = 16
    acPoint3d = 32
    acResbuf = 256
    acString = 4
    acUnknownDataType = 0

class AcValueUnitType(IntEnum):
    acUnitAngle = 2
    acUnitArea = 4
    acUnitDistance = 1
    acUnitVolume = 8
    acUnitless = 0

class AcVerticalAlignment(IntEnum):
    acVerticalAlignmentBaseline = 0
    acVerticalAlignmentBottom = 1
    acVerticalAlignmentMiddle = 2
    acVerticalAlignmentTop = 3

class AcVerticalTextAttachmentType(IntEnum):
    acAttachmentCenter = 0
    acAttachmentLinedCenter = 1

class AcViewportScale(IntEnum):
    acVp100_1 = 18
    acVp10_1 = 17
    acVp1_1 = 2
    acVp1_10 = 7
    acVp1_100 = 13
    acVp1_128in_1ft = 19
    acVp1_16 = 8
    acVp1_16in_1ft = 22
    acVp1_2 = 3
    acVp1_20 = 9
    acVp1_2in_1ft = 28
    acVp1_30 = 10
    acVp1_32in_1ft = 21
    acVp1_4 = 4
    acVp1_40 = 11
    acVp1_4in_1ft = 26
    acVp1_5 = 5
    acVp1_50 = 12
    acVp1_64in_1ft = 20
    acVp1_8 = 6
    acVp1_8in_1ft = 24
    acVp1and1_2in_1ft = 31
    acVp1ft_1ft = 34
    acVp1in_1ft = 30
    acVp2_1 = 14
    acVp3_16in_1ft = 25
    acVp3_32in_1ft = 23
    acVp3_4in_1ft = 29
    acVp3_8in_1ft = 27
    acVp3in_1ft = 32
    acVp4_1 = 15
    acVp6in_1ft = 33
    acVp8_1 = 16
    acVpCustomScale = 1
    acVpScaleToFit = 0

class AcViewportSplitType(IntEnum):
    acViewport2Horizontal = 0
    acViewport2Vertical = 1
    acViewport3Above = 6
    acViewport3Below = 7
    acViewport3Horizontal = 4
    acViewport3Left = 2
    acViewport3Right = 3
    acViewport3Vertical = 5
    acViewport4 = 8

class AcWindowState(IntEnum):
    acMax = 3
    acMin = 2
    acNorm = 1

class AcWireframeType(IntEnum):
    acIsolines = 0
    acIsoparms = 1

class AcXRefDemandLoad(IntEnum):
    acDemandLoadDisabled = 0
    acDemandLoadEnabled = 1
    acDemandLoadEnabledWithCopy = 2

class AcZoomScaleType(IntEnum):
    acZoomScaledAbsolute = 0
    acZoomScaledRelative = 1
    acZoomScaledRelativePSpace = 2

class AcadSecurityParamsConstants(IntEnum):
    ACADSECURITYPARAMS_ALGID_RC4 = 26625

class AcadSecurityParamsType(IntEnum):
    ACADSECURITYPARAMS_ADD_TIMESTAMP = 32
    ACADSECURITYPARAMS_ENCRYPT_DATA = 1
    ACADSECURITYPARAMS_ENCRYPT_PROPS = 2
    ACADSECURITYPARAMS_SIGN_DATA = 16

class AdeskDxfCode(IntEnum):
    kDxfInvalid = -9999
    kDxfXDictionary = -6
    kDxfPReactors = -5
    kDxfOperator = -4
    kDxfXDataStart = -3
    kDxfHeaderId = -2
    kDxfFirstEntId = -2
    kDxfEnd = -1
    kDxfStart = 0
    kDxfText = 1
    kDxfXRefPath = 1
    kDxfShapeName = 2
    kDxfBlockName = 2
    kDxfAttributeTag = 2
    kDxfSymbolTableName = 2
    kDxfMstyleName = 2
    kDxfSymTableRecName = 2
    kDxfAttributePrompt = 3
    kDxfDimStyleName = 3
    kDxfLinetypeProse = 3
    kDxfTextFontFile = 3
    kDxfDescription = 3
    kDxfDimPostStr = 3
    kDxfTextBigFontFile = 4
    kDxfDimAPostStr = 4
    kDxfCLShapeName = 4
    kDxfSymTableRecComments = 4
    kDxfHandle = 5
    kDxfDimBlk = 5
    kDxfDimBlk1 = 6
    kDxfLinetypeName = 6
    kDxfDimBlk2 = 7
    kDxfTextStyleName = 7
    kDxfLayerName = 8
    kDxfCLShapeText = 9
    kDxfXCoord = 10
    kDxfYCoord = 20
    kDxfZCoord = 30
    kDxfElevation = 38
    kDxfThickness = 39
    kDxfReal = 40
    kDxfViewportHeight = 40
    kDxfTxtSize = 40
    kDxfTxtStyleXScale = 41
    kDxfViewWidth = 41
    kDxfViewportAspect = 41
    kDxfTxtStylePSize = 42
    kDxfViewLensLength = 42
    kDxfViewFrontClip = 43
    kDxfViewBackClip = 44
    kDxfShapeXOffset = 44
    kDxfShapeYOffset = 45
    kDxfViewHeight = 45
    kDxfShapeScale = 46
    kDxfPixelScale = 47
    kDxfLinetypeScale = 48
    kDxfDashLength = 49
    kDxfMlineOffset = 49
    kDxfLinetypeElement = 49
    kDxfAngle = 50
    kDxfViewportSnapAngle = 50
    kDxfViewportTwist = 51
    kDxfVisibility = 60
    kDxfViewportGridDisplay = 60
    kDxfLayerLinetype = 61
    kDxfViewportGridMajor = 61
    kDxfColor = 62
    kDxfHasSubentities = 66
    kDxfViewportVisibility = 67
    kDxfViewportActive = 68
    kDxfViewportNumber = 69
    kDxfInt16 = 70
    kDxfViewMode = 71
    kDxfCircleSides = 72
    kDxfViewportZoom = 73
    kDxfViewportIcon = 74
    kDxfViewportSnap = 75
    kDxfViewportGrid = 76
    kDxfViewportSnapStyle = 77
    kDxfViewportSnapPair = 78
    kDxfRegAppFlags = 71
    kDxfTxtStyleFlags = 71
    kDxfLinetypeAlign = 72
    kDxfLinetypePDC = 73
    kDxfInt32 = 90
    kDxfVertexIdentifier = 91
    kDxfSubclass = 100
    kDxfEmbeddedObjectStart = 101
    kDxfControlString = 102
    kDxfDimVarHandle = 105
    kDxfUCSOrg = 110
    kDxfUCSOriX = 111
    kDxfUCSOriY = 112
    kDxfXReal = 140
    kDxfViewBrightness = 141
    kDxfViewContrast = 142
    kDxfInt64 = 160
    kDxfXInt16 = 170
    kDxfNormalX = 210
    kDxfNormalY = 220
    kDxfNormalZ = 230
    kDxfXXInt16 = 270
    kDxfInt8 = 280
    kDxfRenderMode = 281
    kDxfDefaultLightingType = 282
    kDxfShadowFlags = 284
    kDxfBool = 290
    kDxfDefaultLightingOn = 292
    kDxfXTextString = 300
    kDxfBinaryChunk = 310
    kDxfArbHandle = 320
    kDxfSoftPointerId = 330
    kDxfViewBackgroundId = 332
    kDxfShadePlotId = 333
    kDxfLiveSectionId = 334
    kDxfLiveSectionName = 309
    kDxfHardPointerId = 340
    kDxfObjVisualStyleId = 345
    kDxfVpVisualStyleId = 346
    kDxfMaterialId = 347
    kDxfVisualStyleId = 348
    kDxfDragVisualStyleId = 349
    kDxfSoftOwnershipId = 350
    kDxfHardOwnershipId = 360
    kDxfSunId = 361
    kDxfLineWeight = 370
    kDxfPlotStyleNameType = 380
    kDxfPlotStyleNameId = 390
    kDxfXXXInt16 = 400
    kDxfLayoutName = 410
    kDxfColorRGB = 420
    kDxfColorName = 430
    kDxfAlpha = 440
    kDxfGradientObjType = 450
    kDxfGradientPatType = 451
    kDxfGradientTintType = 452
    kDxfGradientColCount = 453
    kDxfGradientAngle = 460
    kDxfGradientShift = 461
    kDxfGradientTintVal = 462
    kDxfGradientColVal = 463
    kDxfGradientName = 470
    kDxfFaceStyleId = 480
    kDxfEdgeStyleId = 481
    kDxfComment = 999
    kDxfXdAsciiString = 1000
    kDxfRegAppName = 1001
    kDxfXdControlString = 1002
    kDxfXdLayerName = 1003
    kDxfXdBinaryChunk = 1004
    kDxfXdHandle = 1005
    kDxfXdXCoord = 1010
    kDxfXdYCoord = 1020
    kDxfXdZCoord = 1030
    kDxfXdWorldXCoord = 1011
    kDxfXdWorldYCoord = 1021
    kDxfXdWorldZCoord = 1031
    kDxfXdWorldXDisp = 1012
    kDxfXdWorldYDisp = 1022
    kDxfXdWorldZDisp = 1032
    kDxfXdWorldXDir = 1013
    kDxfXdWorldYDir = 1023
    kDxfXdWorldZDir = 1033
    kDxfXdReal = 1040
    kDxfXdDist = 1041
    kDxfXdScale = 1042
    kDxfXdInteger16 = 1070
    kDxfXdInteger32 = 1071
    kDxfXdMax = kDxfXdInteger32

class AcEntityTransparency(StrEnum):
    ByLayer = 'ByLayer'
    ByBlock = 'ByBlock'
    attr_name = ...

class uArrowSymbol(StrEnum):
    empty = ''



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from ..Objects import *
from ..AcadEntity import *
from ..Entities import *


class IAcadBlock(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Add3DFace(self, Point1: PyGePoint3d, 
                        Point2: PyGePoint3d, 
                        Point3: PyGePoint3d, 
                        Point4: PyGePoint3d = None) -> Acad3DFace:
        return Acad3DFace(self._obj.Add3DFace(Point1(), Point2(), Point3(), Point4()))

    def Add3DMesh(self, M: int, 
                        N: int, 
                        PointsMatrix: vDoubleArray) -> AcadPolygonMesh:
        return AcadPolygonMesh(self._obj.Add3DMesh(M, N, PointsMatrix()))

    def Add3DPoly(self, PointsArray: PyGePoint3dArray) -> Acad3DPolyline:
        return Acad3DPolyline(self._obj.Add3Dpoly(PointsArray()))
        
    def AddArc(self, Center: PyGePoint3d, 
                        Radius: float, 
                        StartAngle: float, 
                        EndAngle: float) -> AcadArc:
        return AcadArc(self._obj.AddArc(Center(), Radius, StartAngle, EndAngle))

    def AddAttribute(self, Height: float, 
                        Mode: AcAttributeMode, 
                        Prompt: str, 
                        InsertionPoint: PyGePoint3d, 
                        Tag: str, 
                        Value: str) -> AcadAttribute:
        return AcadAttribute(self._obj.AddAttribute(Height, Mode.value, Prompt, InsertionPoint(), Tag, Value))
        
    def AddBox(self, Origin: PyGePoint3d, 
                        Length: float, 
                        Width: float, 
                        Height: float) -> Acad3DSolid:
        return Acad3DSolid(self._obj.AddBox(Origin(), Length, Width, Height))

    def AddCircle(self, Center: PyGePoint3d, 
                        Radius: float) -> AcadCircle:
        return AcadCircle(self._obj.AddCircle(Center(), Radius))
    
    def AddCone(self, Center: PyGePoint3d, 
                        BaseRadius: float, 
                        Height: float) -> Acad3DSolid:
        return Acad3DSolid(self._obj.AddCone(Center(), BaseRadius, Height))

    def AddCustomObject(self, ClassName: str) -> AcadObject:
        return AcadObject(self._obj.AddCustomObject(ClassName))

    def AddCylinder(self, Center: PyGePoint3d, 
                        Radius: float, 
                        Height: float) -> Acad3DSolid:
        return Acad3DSolid(self._obj.AddCylinder(Center(), Radius, Height))

    def AddDim3PointAngular(self, AngleVertex: PyGePoint3d, 
                        FirstEndPoint: PyGePoint3d, 
                        SecondEndPoint: PyGePoint3d, 
                        TextPoint: PyGePoint3d) -> AcadDim3PointAngular:
        return AcadDim3PointAngular(self._obj.AddDim3PointAngular(AngleVertex(), FirstEndPoint(), SecondEndPoint(), TextPoint()))

    def AddDimAligned(self, ExtLine1Point: PyGePoint3d, 
                        ExtLine2Point: PyGePoint3d, 
                        TextPosition: PyGePoint3d) -> AcadDimAligned:
        return AcadDimAligned(self._obj.AddDimAligned(ExtLine1Point(), ExtLine2Point(), TextPosition()))

    def AddDimArc(self, ArcCenter: PyGePoint3d, 
                        FirstEndPoint: PyGePoint3d, 
                        SecondEndPoint: PyGePoint3d, 
                        ArcPoint: PyGePoint3d) -> AcadDimArcLength:
        return AcadDimArcLength(self._obj.AddDimArc(ArcCenter(), FirstEndPoint(), SecondEndPoint(), ArcPoint()))

    def AddDimDiametric(self, ChordPoint: PyGePoint3d, 
                        FarChordPoint: PyGePoint3d, 
                        LeaderLength: float) -> AcadDimDiametric:
        return AcadDimDiametric(self._obj.AddDimDiametric(ChordPoint(), FarChordPoint(), LeaderLength))

    def AddDimOrdinate(self, DefinitionPoint: PyGePoint3d, 
                        LeaderEndPoint: PyGePoint3d, 
                        UseXAxis: bool) -> AcadDimOrdinate:
        return AcadDimOrdinate(self._obj.AddDimOrdinate(DefinitionPoint, LeaderEndPoint, UseXAxis))

    def AddDimRadial(self, Center: PyGePoint3d, 
                        ChordPoint: PyGePoint3d, 
                        LeaderLength: float) -> AcadDimRadial:
        return AcadDimRadial(self._obj.AddDimRadial(Center(), ChordPoint(), LeaderLength))

    def AddDimRadialLarge(self, Center: PyGePoint3d, 
                        ChordPoint: PyGePoint3d, 
                        OverrideCenter: PyGePoint3d, 
                        JogPoint: PyGePoint3d, 
                        JogAngle: float) -> AcadDimRadialLarge:
        return AcadDimRadialLarge(self._obj.AddDimRadialLarge(Center(), ChordPoint(), OverrideCenter(), JogPoint(), JogAngle))

    def AddDimRotated(self, XLine1Point: PyGePoint3d, 
                        XLine2Point: PyGePoint3d, 
                        DimLineLocation: PyGePoint3d, 
                        RotationAngle: float) -> AcadDimRotated:
        return AcadDimRotated(self._obj.AddDimRotated(XLine1Point(), XLine2Point(), DimLineLocation(), RotationAngle))

    def AddEllipse(self, Center: PyGePoint3d, 
                        MajorAxis: PyGeVector3d, 
                        RadiusRatio: float) -> AcadEllipse:
        return AcadEllipse(self._obj.AddEllipse(Center(), MajorAxis(), RadiusRatio))

    def AddEllipticalCone(self, Center: PyGePoint3d, 
                        MajorRadius: float, 
                        MinorRadius: float, 
                        Height: float) -> Acad3DSolid:
        return Acad3DSolid(self._obj.AddEllipticalCone(Center(), MajorRadius, MinorRadius, Height))

    def AddEllipticalCylinder(self, Center: PyGePoint3d, 
                        MajorRadius: float, 
                        MinorRadius: float, 
                        Height: float) -> Acad3DSolid:
        return Acad3DSolid(self._obj.AddEllipticalCylinder(Center(), MajorRadius, MinorRadius, Height))

    def AddExtrudedSolid(self, Profile: AcadRegion, 
                        Height: float, 
                        TaperAngle: float) -> Acad3DSolid:
        return Acad3DSolid(self._obj.AddExtrudedSolid(Profile(), Height, TaperAngle))

    def AddExtrudedSolidAlongPath(self, Profile: AcadRegion, 
                        Path: AcadArc|AcadCircle|AcadEllipse|AcadPolyline|AcadSpline) -> Acad3DSolid:
        return Acad3DSolid(self._obj.AddExtrudedSolidAlongPath(Profile(), Path))

    def AddHatch(self, PatternType: AcPatternType|AcGradientPatternType, 
                        PatternName: str, 
                        Associativity: bool, 
                        HatchObjectType: AcHatchObjectType = None) -> AcadHatch:
        return AcadHatch(self._obj.AddHatch(PatternType, PatternName, Associativity, HatchObjectType))

    def AddLeader(self, PointsArray: PyGePoint3d, 
                        Annotation: AcadBlockReference|AcadMtext|AcadTolerance, 
                        Type: AcLeaderType) -> AcadLeader:
        return AcadLeader(self._obj.AddLeader(PointsArray(), Annotation, Type))

    def AddLightWeightPolyline(self, VerticesList: PyGePoint2dArray) -> AcadLWPolyline:
        return AcadLWPolyline(self._obj.AddLightWeightPolyline(VerticesList()))

    def AddLine(self, StartPoint: PyGePoint3d, 
                        EndPoint: PyGePoint3d) -> AcadLine: 
        return AcadLine(self._obj.AddLine(StartPoint(), EndPoint()))

    def AddMInsertBlock(self, InsertionPoint: PyGePoint3d, 
                        Name: str, 
                        XScale: float, 
                        YScale: float, 
                        ZScale: float, 
                        Rotation: float, 
                        NumRows: int, 
                        NumColumns: int, 
                        RowSpacing: float, 
                        ColumnSpacing: float, 
                        Password: Variant = vObjectEmpty) -> AcadMInsertBlock: 
        return AcadMInsertBlock(self._obj.AddMInsertBlock(InsertionPoint(), Name, XScale, YScale, ZScale, Rotation, NumRows, NumColumns, RowSpacing, ColumnSpacing, Password()))

    def AddMLeader(self, pointsArray: PyGePoint3dArray) -> AcadMLeader: 
        return AcadMLeader(*self._obj.AddMLeader(pointsArray()))

    def AddMLine(self, VertexList: PyGePoint3dArray) -> AcadMLine: 
        return AcadMLine(self._obj.AddMLine(VertexList()))

    def AddMText(self, InsertionPoint: PyGePoint3d, 
                        Width: float, 
                        Text: str) -> AcadMtext: 
        return AcadMtext(self._obj.AddMText(InsertionPoint(), Width, Text))

    def AddPoint(self, Point: PyGePoint3d) -> AcadPoint: 
        return AcadPoint(self._obj.AddPoint(Point()))

    def AddPolyfaceMesh(self, VerticesList: PyGePoint3dArray, 
                        FaceList: vShortArray) -> AcadPolyfaceMesh: 
        return AcadPolyfaceMesh(self._obj.AddPolyfaceMesh(VerticesList(), FaceList()))

    def AddPolyline(self, VerticesList: PyGePoint3dArray) -> AcadPolyline: 
        return AcadPolyline(self._obj.AddPolyline(VerticesList()))

    def AddRaster(self, ImageFileName: str, 
                        InsertionPoint: PyGePoint3d, 
                        ScaleFactor: float, 
                        RotationAngle: float) -> AcadRasterImage: 
        return AcadRasterImage(self._obj.AddRaster(ImageFileName, InsertionPoint(), ScaleFactor, RotationAngle))

    def AddRay(self, Point1: PyGePoint3d, 
                        Point2: PyGePoint3d) -> AcadRay: 
        return AcadRay(self._obj.AddRay(Point1(), Point2()))

    def AddRegion(self, ObjectList: vObjectArray[AcadArc|AcadCircle|AcadEllipse|AcadLine|AcadLWPolyline|AcadSpline]) -> vObjectArray[AcadRegion]: 
        return vObjectArray([AcadRegion(region) for region in self._obj.AddRegion(ObjectList())])

    def AddRevolvedSolid(self, Profile: AcadRegion, 
                        AxisPoint: PyGePoint3d, 
                        AxisDir: PyGeVector3d, 
                        Angle: float) -> Acad3DSolid: 
        return Acad3DSolid(self._obj.AddRevolvedSolid(Profile, AxisPoint(), AxisDir(), Angle))

    def AddSection(self, FromPoint: PyGePoint3d, 
                        ToPoint: PyGePoint3d, 
                        planeVector: PyGeVector3d) -> AcadSection: 
        return AcadSection(self._obj.AddSection(FromPoint(), ToPoint(), planeVector()))

    def AddShape(self, Name: str, 
                        InsertionPoint: PyGePoint3d, 
                        ScaleFactor: float, 
                        Rotation: float) -> AcadShape: 
        return AcadShape(self._obj.AddShape(Name, InsertionPoint(), ScaleFactor, Rotation))

    def AddSolid(self, Point1: PyGePoint3d, 
                        Point2: PyGePoint3d, 
                        Point3: PyGePoint3d, 
                        Point4: PyGePoint3d) -> AcadSolid: 
        return AcadSolid(self._obj.AddSolid(Point1(), Point2(), Point3(), Point4()))

    def AddSphere(self, Center: PyGePoint3d, 
                        Radius: float) -> Acad3DSolid: 
        return Acad3DSolid(self._obj.AddSphere(Center(), Radius))

    def AddSpline(self, PointsArray: PyGePoint3dArray, 
                        StartTangent: PyGeVector3d, 
                        EndTangent: PyGeVector3d) -> AcadSpline: 
        return AcadSpline(self._obj.AddSpline(PointsArray(), StartTangent(), EndTangent()))

    def AddTable(self, InsertionPoint: PyGePoint3d, 
                        NumRows: int, 
                        NumColumns: int, 
                        RowHeight: float, 
                        ColWidth: float) -> AcadTable: 
        return AcadTable(self._obj.AddTable(InsertionPoint(), NumRows, NumColumns, RowHeight, ColWidth))

    def AddText(self, TextString: str, 
                        InsertionPoint: PyGePoint3d, 
                        Height: float) -> AcadText: 
        return AcadText(self._obj.AddText(TextString, InsertionPoint(), Height))

    def AddTolerance(self, Text: str, 
                        InsertionPoint: PyGePoint3d, 
                        Direction: PyGeVector3d) -> AcadTolerance: 
        return AcadTolerance(self._obj.AddTolerance(Text, InsertionPoint(), Direction()))

    def AddTorus(self, Center: PyGePoint3d, 
                        TorusRadius: float, 
                        TubeRadius: float) -> Acad3DSolid: 
        return Acad3DSolid(self._obj.AddTorus(Center(), TorusRadius, TubeRadius))

    def AddTrace(self, PointsArray: PyGePoint3dArray) -> AcadTrace: 
        return AcadTrace(self._obj.AddTrace(PointsArray()))

    def AddWedge(self, Center: PyGePoint3d, 
                        Length: float, 
                        Width: float, 
                        Height: float) -> Acad3DSolid: 
        return Acad3DSolid(self._obj.AddWedge(Center(), Length, Width, Height))

    def AddXline(self, Point1: PyGePoint3d, 
                        Point2: PyGePoint3d) -> AcadXline: 
        return AcadXline(self._obj.AddXline(Point1(), Point2()))

    def AttachExternalReference(self,PathName: str, 
                        Name: str, 
                        InsertionPoint: PyGePoint3d, 
                        XScale: float,
                        YScale: float,
                        ZScale: float,
                        Rotation: float,
                        Overlay: bool,
                        Password: Variant = vObjectEmpty) -> AcadExternalReference: 
        return AcadExternalReference(self._obj.AttachExternalReference(PathName, Name, InsertionPoint(), XScale, YScale, ZScale, Rotation, Overlay, Password()))

    def InsertBlock(self, InsertionPoint: PyGePoint3d, 
                        Name: str, 
                        Xscale: float = 1.0, 
                        Yscale: float = 1.0, 
                        ZScale: float = 1.0, 
                        Rotation: float = 0.0, 
                        Password: Variant = vObjectEmpty) -> AcadBlockReference:
        return AcadBlockReference(self._obj.InsertBlock(InsertionPoint(), Name, Xscale, Yscale, ZScale, Rotation, Password()))

class AcadBlock(IAcadBlock):
    def __init__(self, obj) -> None: super().__init__(obj)

    BlockScaling: AcBlockScaling = proxy_property('AcBlockScaling','BlockScaling',AccessMode.ReadWrite)
    Comments: str = proxy_property(str,'Comments',AccessMode.ReadWrite)
    Explodable: bool = proxy_property(bool,'Explodable',AccessMode.ReadWrite)
    IsDynamicBlock: bool = proxy_property(bool,'IsDynamicBlock',AccessMode.ReadOnly)
    IsLayout: bool = proxy_property(bool,'IsLayout',AccessMode.ReadOnly)
    IsXRef: bool = proxy_property(bool,'IsXRef',AccessMode.ReadOnly)
    Layout: AcadLayout = proxy_property('AcadLayout','Layout',AccessMode.ReadWrite)
    Material: str = proxy_property(str,'Material',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadOnly)
    Origin: PyGePoint3d = proxy_property('PyGePoint3d','Origin',AccessMode.ReadWrite)
    Path: str = proxy_property(str,'Path',AccessMode.ReadWrite)
    Units: AcInsertUnits = proxy_property('AcInsertUnits','Units',AccessMode.ReadWrite)
    XRefDatabase: AcadDatabase = proxy_property('AcadDatabase','XRefDatabase',AccessMode.ReadOnly)

    def Bind(self, bPrefixName: bool) -> None:
        self._obj.Bind(bPrefixName)

    def Delete(self) -> None:
        self._obj.Delete()

    def Detach(self) -> None:
        self._obj.Detach()

    def Reload(self) -> None:
        self._obj.Reload()

    def Unload(self) -> None:
        self._obj.Unload()



from __future__ import annotations
from .Base import *
from .Proxy import *
from .AcadObject import *
from .Objects import *



class AcadEntity(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    EntityTransparency: AcEntityTransparency = proxy_property(str,'EntityTransparency',AccessMode.ReadWrite)
    """Specifies the transparency value for the entity.

    object 
        Type: All drawing objects 
        The object this property applies to. 
    Property Value
        Read-only: No 
        Type: String 
        Use one of the following values: 
        ByLayer: Transparency value determined by layer 
        ByBlock: Transparency value determined by block 
        0: Fully opaque (not transparent) 
        1-90: Transparency value defined as a percentage 
    Remarks
        When representing a percentage of transparency for an entity, the string value is of an integer and not a double. 
    """

    Hyperlinks: AcadHyperlinks = proxy_property('AcadHyperlinks','Hyperlinks',AccessMode.ReadOnly)
    """Gets the Hyperlinks collection for an entity.

    object 
        Type: All drawing objects, AttributeReference, SubDMeshEdge, SubDMeshFace, SubDMeshVertex, SubEntity, SubEntSolidEdge, SubEntSolidFace, SubEntSolidNode, SubEntSolidVertex 
        The objects this property applies to. 
    Property Value
        Read-only: Yes 
        Type: Hyperlinks 
        The Hyperlinks collection for the entity. 
    Remarks
        No additional remarks. 
    """

    Layer: str = proxy_property(str,'Layer',AccessMode.ReadWrite)
    """Specifies the layer for an object.

    object 
        Type: All drawing objects, AttributeReference, Group, SubDMeshEdge, SubDMeshFace, SubDMeshVertex, SubEntity, SubEntSolidEdge, SubEntSolidFace, SubEntSolidNode, SubEntSolidVertex 
        The objects this property applies to. 
    Property Value
        Read-only: No; except for the Group object which is write-only 
        Type: String 
        The name of the layer. 
    Remarks
        All objects have an associated layer. The document always contains at least one layer (layer 0). As with linetypes, you can specify a layer for an object. If you do not specify a layer, the current active layer is used for a new object. If a layer is specified for an object, the current active layer is ignored. Use the ActiveLayer property to set or query the current active layer.
        Each layer has associated properties that can be set and queried through the Layer object. 
    """

    Linetype: str = proxy_property(str,'Linetype',AccessMode.ReadWrite)
    """Specifies the linetype of an object.

    object 
        Type: All drawing objects, AttributeReference, Group, Layer, SubDMeshEdge, SubDMeshFace, SubDMeshVertex, SubEntity, SubEntSolidEdge, SubEntSolidFace, SubEntSolidNode, SubEntSolidVertex 
        The objects this property applies to. 
    Property Value
        Read-only: No; except for Group objects which are write-only 
        Type: String 
        The linetype of an object. The default linetype is the linetype of the layer (ByLayer). 
        Continuous: The default linetype, which is automatically created in the linetype symbol table. 
        ByLayer: The linetype value of the object's layer. 
        ByBlock: The linetype value of the object's surrounding block definition's current block reference. 
    Remarks
        The linetype values identify the series of dots and dashes used for drawing lines. If you do not specify a linetype, the current active linetype is used for a new entity. If a linetype is specified for an entity, the current active linetype is ignored. Use the ActiveLinetype property to set or query the current active linetype.
        NoteIt is not possible to create a linetype programmatically. An existing linetype may be added to a drawing by using the Load method to first load the linetype, and then the Add method to add it to the Linetypes collection. 
    """

    LinetypeScale: float = proxy_property(float,'LinetypeScale',AccessMode.ReadWrite)
    """Specifies the linetype scale of an object.

    object 
        Type: All drawing objects, AttributeReference, Group, SubDMeshEdge, SubDMeshFace, SubDMeshVertex, SubEntity, SubEntSolidEdge, SubEntSolidFace, SubEntSolidNode, SubEntSolidVertex 
        The objects this property applies to. 
    Property Value
        Read-only: No; except for a Group object which is write-only 
        Type: Double 
        This value must be a positive real number. The default is 1.0. 
    Remarks
        The linetype scale of an object specifies the relative length of dash-dot linetypes per drawing unit. 
        ----.----: Linetype scale = 1.0 
        --.--.--.: Linetype scale = 0.5 
        -.-.-.-.-: Linetype scale = 0.25 
    """

    Lineweight: AcLineWeight = proxy_property('AcLineWeight','Lineweight',AccessMode.ReadWrite)
    """Specifies the lineweight of an individual object or the default lineweight for the drawing.

    object 
        Type: All drawing objects, DatabasePreferences, Layer, SubDMeshEdge, SubDMeshFace, SubDMeshVertex, SubEntity, SubEntSolidEdge, SubEntSolidFace, SubEntSolidNode, SubEntSolidVertex 
        The objects this property applies to. 
    Property Value
        Read-only: No 
        Type: acLineWeight enum 
        acLnWtByLayer 
        acLnWtByBlock 
        acLnWtByLwDefault 
        acLnWt000 
        acLnWt005 
        acLnWt009 
        acLnWt013 
        acLnWt015 
        acLnWt018 
        acLnWt020 
        acLnWt025 
        acLnWt030 
        acLnWt035 
        acLnWt040 
        acLnWt050 
        acLnWt053 
        acLnWt060 
        acLnWt070 
        acLnWt080 
        acLnWt090 
        acLnWt100 
        acLnWt106 
        acLnWt120 
        acLnWt140 
        acLnWt158 
        acLnWt200 
        acLnWt211 
    Remarks
        The initial value for this property is acLnWtByBlock. 
        Lineweight values consist of standard settings including BYLAYER, BYBLOCK, and DEFAULT. The DEFAULT value is set by the LWDEFAULT system variable and defaults to a value of 0.01 in. or 0.25 mm. All new objects and layers have a default setting of DEFAULT. The lineweight value of 0 plots at the thinnest lineweight available on the specified plotting device and is displayed at one pixel wide in model space. 
    """

    Material: str = proxy_property(str,'Material',AccessMode.ReadWrite)
    """Specifies the name of the material.

    object 
        Type: All drawing objects, AttributeReference, Group, Layer, SubDMeshFace, SubEntSolidFace 
        The objects this property applies to. 
    Property Value
        Read-only: No 
        Type: String 
        The name of the material. 
    Remarks
        No additional remarks.
    """

    PlotStyleName: str = proxy_property(str,'PlotStyleName',AccessMode.ReadWrite)
    """Specifies the plot style name for an object, group of objects, or layer.

    object 
        Type: All drawing objects, AttributeReference, Dimension, Group, Layer, MLeaderLeader, SubDMeshEdge, SubDMeshFace, SubDMeshVertex, SubEntity, SubEntSolidEdge, SubEntSolidFace, SubEntSolidNode, SubEntSolidVertex 
        The objects this property applies to. 
    Property Value
        Read-only: No; except the Group object which is write-only 
        Type: String 
        The plot style name for the object. 
    Remarks
        No additional remarks.
    """

    TrueColor: AcadAcCmColor = proxy_property('AcadAcCmColor','TrueColor',AccessMode.ReadWrite)
    """Specifies the True Color of an object.

    object 
        Type: All drawing objects, AttributeReference, Dimension, Group, Layer 
        The objects this property applies to. 
    Property Value
        Read-only: No 
        Type: AcCmColor 
        The True Color object of the object. 
    Remarks
        This property is used to change an object's color. Colors are identified by an AcCmColor object. This object can hold an RGB value, an ACI number (an integer from 1 to 255), or a named color. Using an RGB value, you can choose from millions of colors.
    """

    Visible: bool = proxy_property(bool,'Visible',AccessMode.ReadWrite)
    """Specifies the visibility of an object or the application.

    object 
        Type: All drawing objects, Application, AttributeReference, Group, Toolbar 
        The objects this property applies to 
    Property Value
        Read-only: No (except for a Group object which is write-only) 
        Type: Boolean 
        True: The object or application is visible. 
        False: The object or application is not visible. 
    Remarks
        If you specify an object to be invisible, it will be invisible regardless of the application visible setting. Other factors can also cause an object to be invisible; for example, an object will not be displayed if its layer is off or frozen. 
        Specifying the application to be invisible allows you to run tasks in the background without having to see the component.
    """

    def ArrayPolar(self, NumberOfObjects: int, AngleToFill: float, CenterPoint: PyGePoint3d) -> vObjectArray:
        """Creates a polar array of objects given a NumberOfObjects, AngleToFill, and CenterPoint.

        object 
            Type: All drawing objects 
            The objects this method applies to. 
        NumberOfObjects 
            Access: Input-only 
            Type: Long 
            The number of objects to be created in the polar array. This must be a positive integer greater than 1. 
        AngleToFill 
            Access: Input-only 
            Type: Double 
            The angle to fill in radians. A positive value specifies counterclockwise rotation. A negative value specifies clockwise rotation. An error is returned for an angle that equals 0. 
        CenterPoint 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the center point for the polar array. 
        Return Value (RetVal)
            Type: Variant (array of objects) 
            The array of new objects. 
        Remarks
            AutoCAD determines the distance from the array's center point to a reference point on the last object selected. The reference point used depends on the type of object previously selected. AutoCAD uses the center point of a circle or arc, the insertion point of a block or shape, the start point of text, and one endpoint of a line or trace.
            Note that this method does not support the Rotate While Copying option of the AutoCAD ARRAY command.
            NoteYou cannot execute this method while simultaneously iterating through a collection. An iteration will open the work space for a read-only operation, while this method attempts to perform a read-write operation. Complete any iteration before you call this method. 
            AttributeReference: You should not attempt to use this method on AttributeReference objects. AttributeReference objects inherit this method because they are one of the drawing objects, however, it is not feasible to perform this operation on an attribute reference.
        """
        return self._obj.ArrayPolar(NumberOfObjects, AngleToFill, CenterPoint())

    def ArrayRectangular(self, NumberOfRows: int, NumberOfColumns: int, NumberOfLevels: int, DistBetweenRows: float, DistBetweenColumns: float, DistBetweenLevels: float) -> vObjectArray:
        """Creates a 2D or 3D rectangular array of objects.
        
        object 
            Type: All drawing objects 
            The objects this method applies to. 
        NumberOfRows 
            Access: Input-only 
            Type: Long 
            The number of rows in the rectangular array. This must be a positive number. If this number is 1, then NumberOfColumns must be greater than 1. 
        NumberOfColumns 
            Access: Input-only 
            Type: Long 
            The number of columns in the rectangular array. This must be a positive number. If this number is 1, then NumberOfRows must be greater than 1. 
        NumberOfLevels 
            Access: Input-only 
            Type: Long 
            The number of levels in a 3D array. 
        DistBetweenRows 
            Access: Input-only 
            Type: Double 
            The distance between the rows. If the distance between rows is a positive number, rows are added upward from the base entity. If the distance is a negative number, rows are added downward. 
        DistBetweenColumns 
            Access: Input-only 
            Type: Double 
            The distance between the columns. If the distance between columns is a positive number, columns are added to the right of the base entity. If the distance is a negative number, columns are added to the left. 
        DistBetweenLevels 
            Access: Input-only 
            Type: Double 
            The distance between the array levels. If the distance between levels is a positive number, levels are added in the positive direction from the base entity. If the distance is a negative number, levels are added in the negative direction. 
        Return Value (RetVal)
            Type: Variant (array of objects) 
            The array of newly created objects. 
        Remarks
            For a 2D array, specify the NumberOfRows, NumberOfColumns, DistBetweenRow, and DistBetweenColumns. For creating a 3D array, specify the NumberOfLevels and DistBetweenLevels as well. 
            A rectangular array is constructed by replicating the object in the selection set the appropriate number of times. If you define one row, you must specify more than one column and vice versa. 
            The object in the selection set is assumed to be in the lower left-hand corner, and the array is generated up and to the right. If the distance between rows is a negative number, rows are added downward. If the distance between columns is a negative number, the columns are added to the left. 
            AutoCAD builds the rectangular array along a baseline defined by the current snap rotation angle. This angle is zero by default, so the rows and columns of a rectangular array are orthogonal with respect to the X and Y drawing axes. You can change this angle and create a rotated array by setting the snap rotation angle to a nonzero value. To do this, use the SnapRotationAngle property. 
            Note
                You cannot execute this method while simultaneously iterating through a collection. An iteration will open the work space for a read-only operation, while this method attempts to perform a read-write operation. Complete any iteration before you call this method. 
                AttributeReference: You should not attempt to use this method on AttributeReference objects. AttributeReference objects inherit this method because they are one of the drawing objects, however, it is not feasible to perform this operation on an attribute reference.
        """
        return self._obj.ArrayRectangular(NumberOfRows, NumberOfColumns, NumberOfLevels, DistBetweenRows, DistBetweenColumns, DistBetweenLevels)

    def Copy(self) -> AcadEntity:
        """Duplicates the given object to the same location.

        object 
            Type: All drawing objects, AttributeReference, Dimension 
            The objects this method applies to. 
        Return Value (RetVal)
            Type: All drawing objects, AttributeReference, Dimension 
            The newly created duplicate object. 
        Remarks
            AttributeReference: You should not attempt to use this method on AttributeReference objects. AttributeReference objects inherit this method because they are one of the drawing objects, however, it is not feasible to perform this operation on an attribute reference.
            Note
                You cannot execute this method while simultaneously iterating through a collection. An iteration will open the work space for a read-only operation, while this method attempts to perform a read-write operation. Complete any iteration before you call this method.
        """
        return AcadEntity(self._obj.Copy())

    def GetBoundingBox(self) -> PyGePoint3dArray:
        """Gets two points of a box enclosing the specified object.

        object 
            Type: All drawing objects, AttributeReference, Dimension 
            The objects this method applies to. 
        MinPoint 
            Access: Output-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the minimum point of the object's bounding box. 
        MaxPoint 
            Access: Output-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the maximum point of the object's bounding box. 
        Return Value (RetVal)
            No return value.
        """
        return PyGePoint3dArray(self._obj.GetBoundingBox())
    
    def Highlight(self, HighlightFlag: bool) -> None:
        """Sets the highlight status for the given object, or for all objects in a given selection set.

        object 
            Type: All drawing objects, AttributeReference, Dimension, Group, SelectionSet 
            The objects this method applies to. 
        HighlightFlag 
            Access: Input-only 
            Type: Boolean 
            True: The object is highlighted. 
            False: The existing highlight is removed from the object. 
        Return Value (RetVal)
            No return value. 
        Remarks
            Once the highlight flag for an object has been set, a call to the Update or Regen method is required to view the change. 
            Note
                This function does not return the current highlight status of an object. 
        """
        self._obj.Highlight(HighlightFlag)

    def IntersectWith(self, IntersectObject: AcadEntity | AcadAttributeReference, ExtendOption: AcExtendOption) -> PyGePoint3dArray:
        """Gets the points where one object intersects another object in the drawing.

        object 
            Type: All drawing objects (except PViewport and PolygonMesh), AttributeReference 
            The objects this method applies to. 
        IntersectObject 
            Access: Input-only 
            Type: Object 
            The object can be one of the supported drawing objects or an AttributeReference. 
        ExtendOption 
            Access: Input-only 
            Type: AcExtendOption enum 
            This option specifies if none, one or both, of the objects are to be extended in order to attempt an intersection. 
            acExtendNone: Does not extend either object. 
            acExtendThisEntity: Extends the base object. 
            acExtendOtherEntity: Extends the object passed as an argument. 
            acExtendBoth: Extends both objects. 
        Return Value (RetVal)
            Type: Variant (array of doubles) 
            The array of points where one object intersects another object in the drawing. 
        Remarks
            If the two objects do not intersect, no data is returned. You can request the point of intersection that would occur if one or both of the objects were extended to meet the other. For example, in the following illustration, Line1 is the base object from which this method was called and line3 is the object passed as a parameter. If the ExtendOption passed is acExtendThisEntity, point A is returned as the point where line1 would intersect line3 if line1 were extended. If the ExtendOption is acExtendOtherEntity, no data is returned because even if line3 were extended, it would not intersect line1.
            If the intersection type is acExtendBothEntities and line2 is passed as the parameter entity, point B is returned. If the ExtendOption is acExtendNone and line2 is the parameter entity, no data is returned. 
        """
        return self._obj.IntersectWith(IntersectObject(), ExtendOption)

    def Mirror(self, Point1: PyGePoint3d, Point2: PyGePoint3d) -> AcadEntity:
        """Creates a mirror-image copy of a planar object around an axis.

        object 
            Type: All drawing objects, AttributeReference, Dimension 
            The objects this method applies to. 
        Point1 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the first point of the mirror axis. 
        Point2 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the second point of the mirror axis. 
        Return Value (RetVal)
            Type: Object 
            This object can be one of any drawing object and is the result of mirroring the original object. 
        Remarks
            The two points specified as parameters become the endpoints of a line around which the base object is reflected. 
        """
        return self._obj.Mirror(Point1(), Point2())

    def Mirror3D(self, Point1: PyGePoint3d, Point2: PyGePoint3d, Point3: PyGePoint3d) -> AcadEntity:
        """Creates a mirror image of the given object about a plane.

        object 
            Type: All drawing objects, AttributeReference, Dimension 
            The objects this method applies to. 
        Point1 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the first point of the mirror plane. 
        Point2 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the second point of the mirror plane. 
        Point3 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the third point of the mirror plane. 
        Return Value (RetVal)
            Type: Object 
            This object can be one of any drawing object and is the result of mirroring the original object. 
        Remarks
            AutoCAD checks to see if the object to be copied owns any other object. If it does, it performs a copy on those objects as well. The process continues until all owned objects have been copied. 
            Note
                You cannot execute this method while simultaneously iterating through a collection. An iteration will open the work space for a read-only operation, while this method attempts to perform a read-write operation. Complete any iteration before you call this method. 
            AttributeReference: You should not attempt to use this method on AttributeReference objects. AttributeReference objects inherit this method because they are one of the drawing objects, however, it is not feasible to perform this operation on an attribute reference. 

        """
        return self._obj.Mirror3D(Point1(), Point2(), Point3())
    
    def Move(self, Point1: PyGePoint3d, Point2: PyGePoint3d) -> None:
        """Moves an object along a vector.

        object 
            Type: All drawing objects, AttributeReference, Dimension 
            The objects this method applies to. 
        Point1 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the first point of the move vector. 
        Point2 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the second point of the move vector. 
        Return Value (RetVal)
            No return value. 
        Remarks
            The two points you specify define a displacement vector indicating how far the given object is to be moved and in what direction. 
        """
        self._obj.Move(Point1(), Point2())

    def Rotate(self, BasePoint: PyGePoint3d, RotationAngle: float) -> None:
        """Rotates an object around a base point.

        object 
            Type: All drawing objects, AttributeReference, Dimension 
            The objects this method applies to. 
        BasePoint 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the point through which the axis of rotation is defined as parallel to the Z axis of the UCS. 
        RotationAngle 
            Access: Input-only 
            Type: Double 
            The angle in radians to rotate the object. This angle determines how far an object rotates around the base point relative to its current location. 
        No return value. 
        """
        self._obj.Rotate(BasePoint(), RotationAngle)
        
    def Rotate3D(self, Point1: PyGePoint3d, Point2: PyGePoint3d, RotationAngle: float) -> None:
        """Rotates an object around a 3D axis. Point1 and Point2 define the line that becomes the axis of rotation.

        object 
            Type: All drawing objects, AttributeReference, Dimension 
            The object this method applies to. 
        Point1 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the first point of the axis line. 
        Point2 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the second point of the axis line. 
        RotationAngle 
            Access: Input-only 
            Type: Double 
            The angle in radians to rotate the object about the selected axis. 
        Return Value (RetVal)
            No return value. 
        """
        self._obj.Rotate3D(Point1(), Point2(), RotationAngle)
        
    def ScaleEntity(self, BasePoint: PyGePoint3d, ScaleFactor: float) -> None:
        """Scales an object equally in the X, Y, and Z directions.
        
        object 
            Type: All drawing objects, AttributeReference, Dimension 
            The objects this method applies to. 
        BasePoint 
            Access: Input-only 
            Type: Variant (three-element array of doubles) 
            The 3D WCS coordinates specifying the base point. 
        ScaleFactor 
            Access: Input-only 
            Type: Double 
            The factor by which to scale the object. The dimensions of the object are multiplied by the scale factor. A scale factor greater than 1 enlarges the object. A scale factor between 0 and 1 reduces the object. The scale factor must be greater than 0.0.
        Return Value (RetVal)
            No return value. 
        """
        self._obj.ScaleEntity(BasePoint(), ScaleFactor)
        
    def TransformBy(self, TransformationMatrix: PyGeMatrix3d) -> None:
        """Moves, scales, or rotates an object given a 4x4 transformation matrix.

        object 
            Type: All drawing objects, AttributeReference 
            The object this method applies to. 
        TransformationMatrix 
            Access: Input-only 
            Type: Variant (4x4 array of doubles) 
            A 4x4 matrix specifying the transformation to perform. 
        Return Value (RetVal)
            No return value. 
        Remarks
            The following table demonstrates the transformation matrix configuration, where R = Rotation, and T = Translation: R00 
            R00 R01 R02 T0  
            R10 R11 R12 T1  
            R20 R21 R22 T2  
            0   0   0   1  
            This method will return an error if the transformation matrix is not correct. 
            Sample transformation matrices are provided in the example code for this method. 
        """
        self._obj.TransformBy(TransformationMatrix())

    def Update(self) -> None:
        """Updates the object to the drawing screen.
        
        object 
            Type: All drawing objects, Application, AttributeReference, Dimension, SelectionSet 
            The objects this method applies to. 
        Return Value (RetVal)
            No return value. 
        Remarks
            No additional remarks.
        """
        self._obj.Update()


class AcadSubEntity(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Color: AcColor = proxy_property('AcColor','Color',AccessMode.ReadWrite)
    Hyperlinks: AcadHyperlinks = proxy_property('AcadHyperlinks','Hyperlinks',AccessMode.ReadOnly)
    Layer: str = proxy_property(str,'Layer',AccessMode.ReadWrite)
    Linetype: str = proxy_property(str,'Linetype',AccessMode.ReadWrite)
    LinetypeScale: float = proxy_property(float,'LinetypeScale',AccessMode.ReadWrite)
    Lineweight: AcLineWeight = proxy_property('AcLineWeight','Lineweight',AccessMode.ReadWrite)
    ObjectName: str = proxy_property(str,'ObjectName',AccessMode.ReadOnly)
    PlotStyleName: str = proxy_property(str,'PlotStyleName',AccessMode.ReadWrite)



# from enum import StrEnum
# class acTransparency(StrEnum):
#     ByLayer = 'ByLayer'
#     ByBlock = 'ByBlock'


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class Acad3DSolid(AcadEntity):
    def __init__(self, obj) -> None: ...
    Centroid: PyGePoint2d
    History: bool
    MomentOfInertia: PyGePoint3d
    Position: PyGePoint3d
    PrincipalDirections: PyGeVector3d
    PrincipalMoments: PyGeVector3d
    ProductOfInertia: PyGeVector3d
    RadiiOfGyration: PyGeVector3d
    ShowHistory: bool
    SolidType: str
    Volume: float
    def Boolean(self, Operation: AcBooleanType, Object: Acad3DSolid | AcadRegion) -> None: ...
    def CheckInterference(self, Object: Acad3DSolid, CreateInterferenceSolid: bool) -> bool: ...
    def Delete(self) -> None: ...
    def Copy(self) -> Acad3DSolid: ...
    def SectionSolid(self, Point1: PyGePoint3d, Point2: PyGePoint3d, Point3: PyGePoint3d) -> AcadRegion: ...
    def SliceSolid(self, Point1: PyGePoint3d, Point2: PyGePoint3d, Point3: PyGePoint3d, Negative: bool) -> Acad3DSolid: ...



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadGroup(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    Layer: str
    Linetype: str
    LinetypeScale: float
    Lineweight: AcLineWeight
    Material: str
    Name: str
    PlotStyleName: str
    TrueColor: AcadAcCmColor
    Visible: bool
    def AppendItems(self, Objects: vObjectArray) -> None: ...
    def Highlight(self, HighlightFlag: bool) -> None: ...
    def RemoveItems(self, Objects: vObjectArray) -> None: ...
    def Update(self) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from ..Objects import *
from ..AcadEntity import *
from ..Entities import *

class IAcadBlock(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add3DFace(self, Point1: PyGePoint3d, Point2: PyGePoint3d, Point3: PyGePoint3d, Point4: PyGePoint3d = None) -> Acad3DFace: ...
    def Add3DMesh(self, M: int, N: int, PointsMatrix: vDoubleArray) -> AcadPolygonMesh: ...
    def Add3DPoly(self, PointsArray: PyGePoint3dArray) -> Acad3DPolyline: ...
    def AddArc(self, Center: PyGePoint3d, Radius: float, StartAngle: float, EndAngle: float) -> AcadArc: ...
    def AddAttribute(self, Height: float, Mode: AcAttributeMode, Prompt: str, InsertionPoint: PyGePoint3d, Tag: str, Value: str) -> AcadAttribute: ...
    def AddBox(self, Origin: PyGePoint3d, Length: float, Width: float, Height: float) -> Acad3DSolid: ...
    def AddCircle(self, Center: PyGePoint3d, Radius: float) -> AcadCircle: ...
    def AddCone(self, Center: PyGePoint3d, BaseRadius: float, Height: float) -> Acad3DSolid: ...
    def AddCustomObject(self, ClassName: str) -> AcadObject: ...
    def AddCylinder(self, Center: PyGePoint3d, Radius: float, Height: float) -> Acad3DSolid: ...
    def AddDim3PointAngular(self, AngleVertex: PyGePoint3d, FirstEndPoint: PyGePoint3d, SecondEndPoint: PyGePoint3d, TextPoint: PyGePoint3d) -> AcadDim3PointAngular: ...
    def AddDimAligned(self, ExtLine1Point: PyGePoint3d, ExtLine2Point: PyGePoint3d, TextPosition: PyGePoint3d) -> AcadDimAligned: ...
    def AddDimArc(self, ArcCenter: PyGePoint3d, FirstEndPoint: PyGePoint3d, SecondEndPoint: PyGePoint3d, ArcPoint: PyGePoint3d) -> AcadDimArcLength: ...
    def AddDimDiametric(self, ChordPoint: PyGePoint3d, FarChordPoint: PyGePoint3d, LeaderLength: float) -> AcadDimDiametric: ...
    def AddDimOrdinate(self, DefinitionPoint: PyGePoint3d, LeaderEndPoint: PyGePoint3d, UseXAxis: bool) -> AcadDimOrdinate: ...
    def AddDimRadial(self, Center: PyGePoint3d, ChordPoint: PyGePoint3d, LeaderLength: float) -> AcadDimRadial: ...
    def AddDimRadialLarge(self, Center: PyGePoint3d, ChordPoint: PyGePoint3d, OverrideCenter: PyGePoint3d, JogPoint: PyGePoint3d, JogAngle: float) -> AcadDimRadialLarge: ...
    def AddDimRotated(self, XLine1Point: PyGePoint3d, XLine2Point: PyGePoint3d, DimLineLocation: PyGePoint3d, RotationAngle: float) -> AcadDimRotated: ...
    def AddEllipse(self, Center: PyGePoint3d, MajorAxis: PyGeVector3d, RadiusRatio: float) -> AcadEllipse: ...
    def AddEllipticalCone(self, Center: PyGePoint3d, MajorRadius: float, MinorRadius: float, Height: float) -> Acad3DSolid: ...
    def AddEllipticalCylinder(self, Center: PyGePoint3d, MajorRadius: float, MinorRadius: float, Height: float) -> Acad3DSolid: ...
    def AddExtrudedSolid(self, Profile: AcadRegion, Height: float, TaperAngle: float) -> Acad3DSolid: ...
    def AddExtrudedSolidAlongPath(self, Profile: AcadRegion, Path: AcadArc | AcadCircle | AcadEllipse | AcadPolyline | AcadSpline) -> Acad3DSolid: ...
    def AddHatch(self, PatternType: AcPatternType | AcGradientPatternType, PatternName: str, Associativity: bool, HatchObjectType: AcHatchObjectType = None) -> AcadHatch: ...
    def AddLeader(self, PointsArray: PyGePoint3d, Annotation: AcadBlockReference | AcadMtext | AcadTolerance, Type: AcLeaderType) -> AcadLeader: ...
    def AddLightWeightPolyline(self, VerticesList: PyGePoint2dArray) -> AcadLWPolyline: ...
    def AddLine(self, StartPoint: PyGePoint3d, EndPoint: PyGePoint3d) -> AcadLine: ...
    def AddMInsertBlock(self, InsertionPoint: PyGePoint3d, Name: str, XScale: float, YScale: float, ZScale: float, Rotation: float, NumRows: int, NumColumns: int, RowSpacing: float, ColumnSpacing: float, Password: Variant = ...) -> AcadMInsertBlock: ...
    def AddMLeader(self, pointsArray: PyGePoint3dArray) -> AcadMLeader: ...
    def AddMLine(self, VertexList: PyGePoint3dArray) -> AcadMLine: ...
    def AddMText(self, InsertionPoint: PyGePoint3d, Width: float, Text: str) -> AcadMtext: ...
    def AddPoint(self, Point: PyGePoint3d) -> AcadPoint: ...
    def AddPolyfaceMesh(self, VerticesList: PyGePoint3dArray, FaceList: vShortArray) -> AcadPolyfaceMesh: ...
    def AddPolyline(self, VerticesList: PyGePoint3dArray) -> AcadPolyline: ...
    def AddRaster(self, ImageFileName: str, InsertionPoint: PyGePoint3d, ScaleFactor: float, RotationAngle: float) -> AcadRasterImage: ...
    def AddRay(self, Point1: PyGePoint3d, Point2: PyGePoint3d) -> AcadRay: ...
    def AddRegion(self, ObjectList: vObjectArray[AcadArc | AcadCircle | AcadEllipse | AcadLine | AcadLWPolyline | AcadSpline]) -> vObjectArray[AcadRegion]: ...
    def AddRevolvedSolid(self, Profile: AcadRegion, AxisPoint: PyGePoint3d, AxisDir: PyGeVector3d, Angle: float) -> Acad3DSolid: ...
    def AddSection(self, FromPoint: PyGePoint3d, ToPoint: PyGePoint3d, planeVector: PyGeVector3d) -> AcadSection: ...
    def AddShape(self, Name: str, InsertionPoint: PyGePoint3d, ScaleFactor: float, Rotation: float) -> AcadShape: ...
    def AddSolid(self, Point1: PyGePoint3d, Point2: PyGePoint3d, Point3: PyGePoint3d, Point4: PyGePoint3d) -> AcadSolid: ...
    def AddSphere(self, Center: PyGePoint3d, Radius: float) -> Acad3DSolid: ...
    def AddSpline(self, PointsArray: PyGePoint3dArray, StartTangent: PyGeVector3d, EndTangent: PyGeVector3d) -> AcadSpline: ...
    def AddTable(self, InsertionPoint: PyGePoint3d, NumRows: int, NumColumns: int, RowHeight: float, ColWidth: float) -> AcadTable: ...
    def AddText(self, TextString: str, InsertionPoint: PyGePoint3d, Height: float) -> AcadText: ...
    def AddTolerance(self, Text: str, InsertionPoint: PyGePoint3d, Direction: PyGeVector3d) -> AcadTolerance: ...
    def AddTorus(self, Center: PyGePoint3d, TorusRadius: float, TubeRadius: float) -> Acad3DSolid: ...
    def AddTrace(self, PointsArray: PyGePoint3dArray) -> AcadTrace: ...
    def AddWedge(self, Center: PyGePoint3d, Length: float, Width: float, Height: float) -> Acad3DSolid: ...
    def AddXline(self, Point1: PyGePoint3d, Point2: PyGePoint3d) -> AcadXline: ...
    def AttachExternalReference(self, PathName: str, Name: str, InsertionPoint: PyGePoint3d, XScale: float, YScale: float, ZScale: float, Rotation: float, Overlay: bool, Password: Variant = ...) -> AcadExternalReference: ...
    def InsertBlock(self, InsertionPoint: PyGePoint3d, Name: str, Xscale: float = 1.0, Yscale: float = 1.0, ZScale: float = 1.0, Rotation: float = 0.0, Password: Variant = ...) -> AcadBlockReference: ...

class AcadBlock(IAcadBlock):
    def __init__(self, obj) -> None: ...
    BlockScaling: AcBlockScaling
    Comments: str
    Explodable: bool
    IsDynamicBlock: bool
    IsLayout: bool
    IsXRef: bool
    Layout: AcadLayout
    Material: str
    Name: str
    Origin: PyGePoint3d
    Path: str
    Units: AcInsertUnits
    XRefDatabase: AcadDatabase
    def Bind(self, bPrefixName: bool) -> None: ...
    def Delete(self) -> None: ...
    def Detach(self) -> None: ...
    def Reload(self) -> None: ...
    def Unload(self) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadPlotConfiguration import *

class AcadPlotConfigurations(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add(self, Name: str, ModelType: bool = None) -> AcadPlotConfiguration: ...
    def Item(self, Index: int | str) -> AcadPlotConfiguration: ...
    def __iter__(self): ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadSurface import AcadSurface



class AcadNurbSurface(AcadSurface):
    def __init__(self, obj) -> None: super().__init__(obj)

    CvHullDisplay: bool = proxy_property(bool,'CvHullDisplay',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)

    def Copy(self) -> AcadNurbSurface:
        return AcadNurbSurface(self._obj.Copy())


from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadPlot(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    BatchPlotProgress: bool
    NumberOfCopies: int
    QuietErrorMode: bool
    def DisplayPlotPreview(self, Preview: AcPreviewMode) -> None: ...
    def PlotToDevice(self, plotConfig: str = '') -> bool: ...
    def PlotToFile(self, plotFile: str, plotConfig: str = '') -> bool: ...
    def SetLayoutsToPlot(self, layoutList: vStringArray) -> None: ...
    def StartBatchMode(self, entryCount: int) -> None: ...



from ..Base import *
from ..Proxy import *
from .AcadPopupMenu import *

class AcadPopupMenuItem(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Caption: str
    Check: bool
    Enable: bool
    EndSubMenuLevel: int
    HelpString: str
    Index: str
    Label: str
    Parent: AppObject
    SubMenu: AcadPopupMenu
    TagString: str
    Type: AcMenuItemType
    def Delete(self) -> None: ...



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadLineTypes(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add(self, Name: str) -> AcadLineType: ...
    def Item(self, Index: int | str) -> AcadLineType: ...
    def Load(self, LineTypeName: str, FileName: str) -> None: ...
    def __iter__(self): ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadBlock import IAcadBlock


class AcadModelSpace(IAcadBlock):
    def __init__(self, obj) -> None: super().__init__(obj)

    Comments: str = proxy_property(str,'Comments',AccessMode.ReadWrite)
    Layout: AcadLayout = proxy_property('AcadLayout','Layout',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadOnly)
    Origin: PyGePoint3d = proxy_property('PyGePoint3d','Origin',AccessMode.ReadWrite)
    Units: AcInsertUnits = proxy_property('AcInsertUnits','Units',AccessMode.ReadWrite)



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadDimension import AcadDimension



class AcadDim3PointAngular(AcadDimension):
    def __init__(self, obj) -> None: super().__init__(obj)

    AngleFormat: AcAngleUnits = proxy_property('AcAngleUnits','AngleFormat',AccessMode.ReadWrite)
    AngleVertex: PyGePoint3d = proxy_property('PyGePoint3d','AngleFormat',AccessMode.ReadWrite)
    Arrowhead1Block: str = proxy_property(str,'Arrowhead1Block',AccessMode.ReadWrite)
    Arrowhead1Type: AcDimArrowheadType = proxy_property('AcDimArrowheadType','Arrowhead1Type',AccessMode.ReadWrite)
    Arrowhead2Block: str = proxy_property(str,'Arrowhead2Block',AccessMode.ReadWrite)
    Arrowhead2Type: AcDimArrowheadType = proxy_property('AcDimArrowheadType','Arrowhead2Type',AccessMode.ReadWrite)
    ArrowheadSize: int = proxy_property(int,'ArrowheadSize',AccessMode.ReadWrite)
    # ArrowSize: int = proxy_property(int,'ArrowSize',AccessMode.ReadWrite)
    DimConstrDesc: str = proxy_property(str,'DimConstrDesc',AccessMode.ReadWrite)
    DimConstrExpression: str = proxy_property(str,'DimConstrExpression',AccessMode.ReadWrite)
    DimConstrForm: bool = proxy_property(bool,'DimConstrForm',AccessMode.ReadWrite)
    DimConstrName: str = proxy_property(str,'DimConstrName',AccessMode.ReadWrite)
    DimConstrReference: bool = proxy_property(bool,'DimConstrReference',AccessMode.ReadWrite)
    DimConstrValue: str = proxy_property(str,'DimConstrValue',AccessMode.ReadWrite)
    DimensionLineColor: AcColor = proxy_property('AcColor','DimensionLineColor',AccessMode.ReadWrite)
    DimensionLinetype: str = proxy_property(str,'DimensionLinetype',AccessMode.ReadWrite)
    DimensionLineWeight: AcLineWeight = proxy_property('AcLineWeight','DimensionLineWeight',AccessMode.ReadWrite)
    DimLine1Suppress: bool = proxy_property(bool,'DimLine1Suppress',AccessMode.ReadWrite)
    DimLine2Suppress: bool = proxy_property(bool,'DimLine2Suppress',AccessMode.ReadWrite)
    DimLineInside: bool = proxy_property(bool,'DimLineInside',AccessMode.ReadWrite)
    ExtensionLineColor: AcColor = proxy_property('AcColor','ExtensionLineColor',AccessMode.ReadWrite)
    ExtensionLineExtend: float = proxy_property(float,'ExtensionLineExtend',AccessMode.ReadWrite)
    ExtensionLineOffset: float = proxy_property(float,'ExtensionLineOffset',AccessMode.ReadWrite)
    ExtensionLineWeight: AcLineWeight = proxy_property('AcLineWeight','ExtensionLineWeight',AccessMode.ReadWrite)
    ExtLine1EndPoint: PyGePoint3d = proxy_property('PyGePoint3d','ExtLine1EndPoint',AccessMode.ReadWrite)
    ExtLine1Linetype: str = proxy_property(str,'ExtLine1Linetype',AccessMode.ReadWrite)
    ExtLine1Suppress: bool = proxy_property(bool,'ExtLine1Suppress',AccessMode.ReadWrite)
    ExtLine2EndPoint: PyGePoint3d = proxy_property('PyGePoint3d','ExtLine2EndPoint',AccessMode.ReadWrite)
    ExtLine2Linetype: str = proxy_property(str,'ExtLine2Linetype',AccessMode.ReadWrite)
    ExtLine2Suppress: bool = proxy_property(bool,'ExtLine2Suppress',AccessMode.ReadWrite)
    ExtLineFixedLen: float = proxy_property(float,'ExtLineFixedLen',AccessMode.ReadWrite)
    ExtLineFixedLenSuppress: bool = proxy_property(bool,'ExtLineFixedLenSuppress',AccessMode.ReadWrite)
    Fit: AcDimFit = proxy_property('AcDimFit','Fit',AccessMode.ReadWrite)
    ForceLineInside: bool = proxy_property(bool,'ForceLineInside',AccessMode.ReadWrite)
    HorizontalTextPosition: AcDimHorizontalJustification = proxy_property('AcDimHorizontalJustification','HorizontalTextPosition',AccessMode.ReadWrite)
    Measurement: float = proxy_property(float,'Measurement',AccessMode.ReadOnly)
    TextInside: bool = proxy_property(bool,'TextInside',AccessMode.ReadWrite)
    TextInsideAlign: bool = proxy_property(bool,'TextInsideAlign',AccessMode.ReadWrite)
    TextOutsideAlign: bool = proxy_property(bool,'TextOutsideAlign',AccessMode.ReadWrite)
    TextPrecision: AcDimPrecision = proxy_property('AcDimPrecision','TextPrecision',AccessMode.ReadWrite)

    def Copy(self) -> AcadDim3PointAngular:
        return AcadDim3PointAngular(self._obj.Copy())


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadLeader(AcadEntity):
    def __init__(self, obj) -> None: ...
    Annotation: AcadBlockReference | AcadMtext | AcadTolerance
    ArrowheadBlock: str
    ArrowheadSize: float
    ArrowheadType: AcDimArrowheadType
    Coordinates: PyGePoint3dArray
    DimensionLineColor: AcColor
    DimensionLineWeight: AcLineWeight
    Normal: PyGeVector3d
    TextGap: float
    Type: AcLeaderType
    def Coordinate(self, Index: int) -> PyGePoint3d: ...
    def Copy(self) -> AcadLeader: ...
    def Evaluate(self) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadMLine(AcadEntity):
    def __init__(self, obj) -> None: ...
    Coordinates: PyGePoint3d
    Justification: AcMLineJustification
    MLineScale: float
    StyleName: str
    def Copy(self) -> AcadMtext: ...



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferencesOpenSave(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    AutoAudit: bool
    AutoSaveInterval: int
    CreateBackup: bool
    DemandLoadARXApp: AcARXDemandLoad
    FullCRCValidation: bool
    IncrementalSavePercent: int
    LogFileOn: bool
    MRUNumber: int
    ProxyImage: AcProxyImage
    SaveAsType: AcSaveAsType
    SavePreviewThumbnail: bool
    ShowProxyDialogBox: bool
    TempFileExtension: str
    XRefDemandLoad: AcXRefDemandLoad



from ..Base import *
from ..Proxy import *
from .AcadUnderlay import AcadUnderlay as AcadUnderlay

class AcadPdfUnderlay(AcadUnderlay):
    def __init__(self, obj) -> None: ...
    def Copy(self) -> AcadPdfUnderlay: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadUnderlay import AcadUnderlay



class AcadPdfUnderlay(AcadUnderlay):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Copy(self) -> AcadPdfUnderlay: 
        return AcadPdfUnderlay(self._obj.Copy())



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferencesSelection(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    DisplayGrips: bool
    DisplayGripsWithinBlocks: bool
    GripColorSelected: AcColor
    GripColorUnselected: AcColor
    GripSize: int
    PickAdd: bool
    PickAuto: bool
    PickBoxSize: int
    PickDrag: bool
    PickFirst: bool
    PickGroup: bool



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadPlotConfiguration(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    CanonicalMediaName: str = proxy_property(str,'CanonicalMediaName',AccessMode.ReadWrite)
    CenterPlot: bool = proxy_property(bool,'CenterPlot',AccessMode.ReadWrite)
    ConfigName: str = proxy_property(str,'ConfigName',AccessMode.ReadWrite)
    ModelType: bool = proxy_property(bool,'ModelType',AccessMode.ReadOnly)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    PaperUnits: AcPlotPaperUnits = proxy_property('AcPlotPaperUnits','PaperUnits',AccessMode.ReadWrite)
    PlotHidden: bool = proxy_property(bool,'PlotHidden',AccessMode.ReadWrite)
    PlotOrigin: PyGePoint2d = proxy_property('PyGePoint2d','PlotOrigin',AccessMode.ReadWrite)
    PlotRotation: AcPlotRotation = proxy_property('AcPlotRotation','PlotRotation',AccessMode.ReadWrite)
    PlotType: AcPlotType = proxy_property('AcPlotType','PlotType',AccessMode.ReadWrite)
    PlotViewportBorders: bool = proxy_property(bool,'PlotViewportBorders',AccessMode.ReadWrite)
    PlotViewportsFirst: bool = proxy_property(bool,'PlotViewportsFirst',AccessMode.ReadWrite)
    PlotWithLineweights: bool = proxy_property(bool,'PlotWithLineweights',AccessMode.ReadWrite)
    PlotWithPlotStyles: bool = proxy_property(bool,'PlotWithPlotStyles',AccessMode.ReadWrite)
    ScaleLineweights: bool = proxy_property(bool,'ScaleLineweights',AccessMode.ReadWrite)
    ShowPlotStyles: bool = proxy_property(bool,'ShowPlotStyles',AccessMode.ReadWrite)
    StandardScale: AcPlotScale = proxy_property('AcPlotScale','StandardScale',AccessMode.ReadWrite)
    StyleSheet: str = proxy_property(str,'StyleSheet',AccessMode.ReadWrite)
    UseStandardScale: bool = proxy_property(bool,'UseStandardScale',AccessMode.ReadWrite)
    ViewToPlot: str = proxy_property(str,'ViewToPlot',AccessMode.ReadWrite)

    def CopyFrom(self, SourceObject: AcadPlotConfiguration) -> None:
        self._obj.CopyFrom(SourceObject)

    def GetCanonicalMediaNames(self) -> tuple[str]:
        return self._obj.GetCanonicalMediaNames()

    def GetCustomScale(self) -> vDoubleArray:
        Numerator, Denominator = self._obj.GetCustomScale()
        return vDoubleArray(Numerator, Denominator)

    def GetLocaleMediaName(self, Name: str) -> str: 
        return self._obj.GetLocaleMediaName(Name)

    def GetPaperMargins(self) -> vDoubleArray:
        LowerLeft, UpperRight = self._obj.GetPaperMargins()
        return vDoubleArray(LowerLeft, UpperRight)

    def GetPaperSize(self) -> tuple:
        Width, Height = self._obj.GetPaperSize()
        return Width, Height

    def GetPlotDeviceNames(self) -> tuple[str]:
        return self._obj.GetPlotDeviceNames()

    def GetPlotStyleTableNames(self) -> tuple[str]:
        return self._obj.GetPlotStyleTableNames()

    def GetWindowToPlot(self) -> PyGePoint2dArray:
        LowerLeft, UpperRight = self._obj.GetWindowToPlot()
        return PyGePoint2dArray(LowerLeft, UpperRight)

    def RefreshPlotDeviceInfo(self) -> None:
        self._obj.RefreshPlotDeviceInfo()

    def SetCustomScale(self, Numerator: float, Denominator: float) -> None:
        self._obj.SetCustomScale(Numerator, Denominator)
        
    def SetWindowToPlot(self, LowerLeft: PyGePoint2d, UpperRight: PyGePoint2d) -> None:
        self._obj.SetWindowToPlot(LowerLeft(), UpperRight())



from __future__ import annotations
from ..Base import *
from ..Proxy import *
# from ..AcadObject import *

class AcadDatabasePreferences(AppObject):
    def __init__(self, obj): super().__init__(obj)

    AllowLongSymbolNames: bool = proxy_property(bool,'AllowLongSymbolNames',AccessMode.ReadWrite)
    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    ContourLinesPerSurface: int = proxy_property(int,'ContourLinesPerSurface',AccessMode.ReadWrite)
    DisplaySilhouette: bool = proxy_property(bool,'DisplaySilhouette',AccessMode.ReadWrite)
    Lineweight: AcLineWeight = proxy_property('AcLineWeight','Lineweight',AccessMode.ReadWrite)
    LineweightDisplay: bool = proxy_property(bool,'LineweightDisplay',AccessMode.ReadWrite)
    ObjectSortByPlotting: bool = proxy_property(bool,'ObjectSortByPlotting',AccessMode.ReadWrite)
    ObjectSortByPSOutput: bool = proxy_property(bool,'ObjectSortByPSOutput',AccessMode.ReadWrite)
    ObjectSortByRedraws: bool = proxy_property(bool,'ObjectSortByRedraws',AccessMode.ReadWrite)
    ObjectSortByRegens: bool = proxy_property(bool,'ObjectSortByRegens',AccessMode.ReadWrite)
    ObjectSortBySelection: bool = proxy_property(bool,'ObjectSortBySelection',AccessMode.ReadWrite)
    ObjectSortBySnap: bool = proxy_property(bool,'ObjectSortBySnap',AccessMode.ReadWrite)
    OLELaunch: bool = proxy_property(bool,'OLELaunch',AccessMode.ReadWrite)
    RenderSmoothness: float = proxy_property(float,'RenderSmoothness',AccessMode.ReadWrite)
    SegmentPerPolyline: int = proxy_property(int,'SegmentPerPolyline',AccessMode.ReadWrite)
    SolidFill: bool = proxy_property(bool,'SolidFill',AccessMode.ReadWrite)
    TextFrameDisplay: bool = proxy_property(bool,'TextFrameDisplay',AccessMode.ReadWrite)
    XRefEdit: bool = proxy_property(bool,'XRefEdit',AccessMode.ReadWrite)
    XRefLayerVisibility: bool = proxy_property(bool,'XRefLayerVisibility',AccessMode.ReadWrite)
    


from ..Proxy import *
from ..AcadObject import AcadObject as AcadObject
from .AcadXRecord import AcadXRecord as AcadXRecord

class AcadDictionary(AcadObject):
    def __init__(self, obj) -> None: ...
    Name: str
    Count: int
    def AddObject(self, Keyword: str, ObjectName: str) -> AcadObject: ...
    def AddXRecord(self, Keyword: str) -> AcadXRecord: ...
    def GetName(self, Object: AppObject) -> str: ...
    def GetObject(self, Name: str) -> AcadObject: ...
    def Item(self, Index: int | str) -> AcadObject: ...
    def Remove(self, Name: str) -> AcadObject: ...
    def Rename(self, OldName: str, NewName: str) -> None: ...
    def Replace(self, Name: str, NewObject: AppObject) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadOle(AcadEntity):
    def __init__(self, obj) -> None: ...
    Height: float
    InsertionPoint: PyGePoint3d
    LockAspectRatio: bool
    OleItemType: AcOleType
    OlePlotQuality: AcOlePlotQuality
    OleSourceApp: str
    Rotation: float
    ScaleHeight: float
    ScaleWidth: float
    Width: float
    def Copy(self) -> AcadOle: ...



from _typeshed import Incomplete
from enum import IntEnum
from win32com.client import VARIANT

def is_iterable(obj): ...
def flatten_generic(nested_iterable): ...

class VT(IntEnum):
    BOOL = ...
    FLOAT = ...
    DOUBLE = ...
    LONG = ...
    INT = ...
    SHORT = ...
    STRING = ...
    OBJECT = ...
    VARIANT = ...
    EMPTY = ...
    NULL = ...
    ARRAY = ...
    ARRAY_BOOL = BOOL | ARRAY
    ARRAY_FLOAT = FLOAT | ARRAY
    ARRAY_DOUBLE = DOUBLE | ARRAY
    ARRAY_LONG = LONG | ARRAY
    ARRAY_INT = INT | ARRAY
    ARRAY_SHORT = SHORT | ARRAY
    ARRAY_STRING = STRING | ARRAY
    ARRAY_OBJECT = OBJECT | ARRAY
    ARRAY_VARIANT = VARIANT | ARRAY

class Variant:
    VT_TYPE_MAP: Incomplete
    def __init__(self, value, force_vt_type: VT = None) -> None: ...
    @property
    def value(self): ...
    @property
    def variant(self): ...
    @property
    def varianttype(self) -> VT: ...
    @property
    def valuetype(self) -> type: ...
    def set_iterable(self, is_iterable: bool = True): ...
    def __iter__(self): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value) -> None: ...
    def __len__(self) -> int: ...
    def to_variant(self, force_vt_type: VT = None): ...
    def __call__(self, force_vt_type=None): ...

vObjectEmpty: Incomplete

class vObject(Variant):
    def __init__(self, obj) -> None: ...

class vDouble(Variant):
    def __init__(self, double) -> None: ...

class vLong(Variant):
    def __init__(self, long) -> None: ...

class vInteger(Variant):
    def __init__(self, integer) -> None: ...

class vShort(Variant):
    def __init__(self, boolean) -> None: ...

class vBool(Variant):
    def __init__(self, boolean) -> None: ...

class vString(Variant):
    def __init__(self, string) -> None: ...

class vArray(Variant):
    def __init__(self, *args, data_type: VT, check_count, set_default) -> None: ...

class vObjectArray(vArray):
    def __init__(self, *args, check_count=None, set_default=None) -> None: ...

class vVariantArray(vArray):
    def __init__(self, *args, check_count=None, set_default=None) -> None: ...

class vDoubleArray(vArray):
    def __init__(self, *args, check_count=None, set_default=None) -> None: ...

class vLongArray(vArray):
    def __init__(self, *args, check_count=None, set_default=None) -> None: ...

class vStringArray(vArray):
    def __init__(self, *args, check_count=None, set_default=None) -> None: ...

class vIntegerArray(vArray):
    def __init__(self, *args, check_count=None, set_default=None) -> None: ...

class vBoolArray(vArray):
    def __init__(self, *args, check_count=None, set_default=None) -> None: ...

class vShortArray(vArray):
    def __init__(self, *args, check_count=None, set_default=None) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadPolygonMesh(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Coordinates: PyGePoint3dArray = proxy_property('PyGePoint3dArray','Coordinates',AccessMode.ReadWrite)
    MClose: bool = proxy_property(bool,'MClose',AccessMode.ReadWrite)
    MDensity: int = proxy_property(int,'MDensity',AccessMode.ReadWrite)
    MVertexCount: int = proxy_property(int,'MVertexCount',AccessMode.ReadOnly)
    NClose: bool = proxy_property(bool,'NClose',AccessMode.ReadWrite)
    NDensity: int = proxy_property(int,'NDensity',AccessMode.ReadWrite)
    NVertexCount: int = proxy_property(int,'NVertexCount',AccessMode.ReadOnly)
    Type: AcPolymeshType   = proxy_property('AcPolymeshType','Type',AccessMode.ReadWrite)


    def Coordinate(self, Index: int) -> PyGePoint3d:
        return PyGePoint3d(self._obj.Coordinate(Index))
        
    def Copy(self) -> AcadPolygonMesh:
        return AcadPolygonMesh(self._obj.Copy())
        
    def AppendVertex(self, Point: PyGePoint3d) -> None:
        self._obj.AppendVertex(Point())
        
    def Explode(self) -> vObjectArray:
        return vObjectArray(self._obj.Explode())



from win32com.client import GetActiveObject as AppAttach, Dispatch as AppCreate, CDispatch
from winreg import CloseKey, OpenKey, QueryValueEx, HKEY_CLASSES_ROOT
from pythoncom import CoInitialize, CoUninitialize, CoCreateInstance, CLSCTX_LOCAL_SERVER, IID_IDispatch, com_error
from abc import ABC, abstractmethod

def com_server_is_running(prog_id: str):
    try:
        return AppAttach(prog_id)
    except com_error as e:
        return False
        raise com_error(e)

def get_clsid(cls):
    if cls.__app_version__:
        curver = f'{cls.__app_name__}.Application.{cls.__app_version__}'
        registry_path_clsid = r"{}\CLSID".format(curver)
    else:
        registry_path_clsid = r"{}.Application\CLSID".format(cls.__app_name__)
        registry_path_curver = r"{}.Application\CurVer".format(cls.__app_name__)
    try:
        key = OpenKey(HKEY_CLASSES_ROOT, registry_path_clsid)
        clsid = QueryValueEx(key, None)[0]
        CloseKey(key)
        if not cls.__app_version__:
            key = OpenKey(HKEY_CLASSES_ROOT, registry_path_curver)
            curver = QueryValueEx(key, None)[0]
            CloseKey(key)
            try:
                cls.__app_version__ = str(curver).split('.')[2]
            except: pass
        return clsid, curver
    except FileNotFoundError:
        raise FileNotFoundError(f"Ключ реестра '{registry_path_clsid}' не найден. Приложение не установлено или COM не зарегистрирован.")
    except Exception as e:
        raise Exception(f"Произошла ошибка при чтении реестра: {e}")

def create_new_instance_explicitly(clsid):
    try:
        obj = CoCreateInstance(
            clsid,
            None,
            CLSCTX_LOCAL_SERVER,
            IID_IDispatch
        )
        app = AppCreate(obj)
        return app
    except com_error as e:
        raise com_error(f"Ошибка COM при создании экземпляра приложения: {e}")

class AppObject:
    def __init__(self, obj):
        if not isinstance(obj,CDispatch): 
            obj = obj._obj
        self._obj = obj
    def __repr__(self):
        return self._obj.__class__
    def __str__(self):
        return f'{self._obj}'
    def __getattr__(self, name):
        return getattr(self._obj,name)
    def __call__(self):
        return self._obj

class Application(ABC):
    def __new__(cls, dispatch_object=None):
        instance = super().__new__(cls)
        instance._dispatch_obj_to_init = None 
        instance._is_owner = False
        instance.__app_version__ = None
        instance.__app_clsid__ = None
        instance.__app_full_name__ = None
        # Логика определения типа подключения
        if dispatch_object is not None:
            instance._dispatch_obj_to_init = dispatch_object
        else:
            try:
                instance._dispatch_obj_to_init = instance._manage_application_instance()
            except Exception as e:
                Exception(f"Ошибка запуска приложения: {e}")
        return instance
    
    def __init__(self, dispatch_object=None):
        super().__init__(self._dispatch_obj_to_init)
        del self._dispatch_obj_to_init # Очищаем временное хранилище

    @abstractmethod
    def _manage_application_instance(self): pass
    


class AppObjectCollection(AppObject):
    def __init__(self, obj):
        super().__init__(obj)

    @property
    def Count(self) -> int: 
        return self._obj.Count

    def Item(self, Index: int|str) -> AppObject:
        return AppObject(self._obj.Item(Index))

    def __iter__(self):
        for item in self._obj:
            obj = AppObject(item)
            yield obj

def clear_com_cache():
    import win32com.client.gencache
    import os
    import shutil
    gentype_path = win32com.client.gencache.GetGeneratePath()
    if os.path.exists(gentype_path):
        # logger.debug(f"Удаление кэша COM: {gentype_path}")
        try:
            shutil.rmtree(gentype_path)
            # logger.debug("Кэш успешно удален.")
        except OSError as e:
            raise com_error(f"Ошибка при удалении кэша: {e}. Закройте приложение COM-сервер и повторите попытку.")
    else:
        pass
        # logger.debug("Папка кэша gen_py не найдена. Возможно, кэш еще не был создан.")


from __future__ import annotations
from ..VarType import *
from .Matrix import *
from .Vector import *
import math

class PyGePoint3d(vDoubleArray):
    """3d точка"""
    def __init__(self, *args):
        """x=0.0, y=0.0, z=0.0"""
        super().__init__(*args, check_count = 3, set_default = 0.0)

    @property
    def x(self): return self[0]
    @x.setter
    def x(self, value): self[0] = value
    @property
    def y(self): return self[1]
    @y.setter
    def y(self, value): self[1] = value
    @property
    def z(self): return self[2]
    @z.setter
    def z(self, value): self[2] = value

    def __add__(self, vector: PyGeVector3d) -> PyGePoint3d:
        """Сложение точки с вектором (Point + Vector)"""
        if isinstance(vector, PyGeVector3d):
            return PyGePoint3d(self.x + vector.x,self.y + vector.y, self.z + vector.z)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")
    
    def __sub__(self, other: PyGePoint3d|PyGeVector3d) -> PyGeVector3d|PyGePoint3d:
        """
        Вычитание:
        1. Точка - Точка = Вектор (Point - Point = Vector)
        2. Точка - Вектор = Точка (Point - Vector = Point)
        """
        if isinstance(other, PyGePoint3d):
            return PyGeVector3d(self.x - other.x, self.y - other.y, self.x - other.z)
        elif isinstance(other, PyGeVector3d):
            return PyGePoint3d(self.x - other.x, self.y - other.y, self.z - other.z)
        raise TypeError(f"Неподдерживаемый тип операнда {type(other)}")

    def __mul__(self, scalar: int|float) -> PyGePoint3d:
        """Умножение точки на скаляр (Point * Scalar)"""
        if isinstance(scalar, (int,float)):
            return PyGePoint3d(self.x * scalar, self.y * scalar, self.z * scalar)
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")

    def __rmul__(self, scalar: int|float) -> PyGePoint3d:
        """Зеркальное умножение на скаляр (Scalar * Point)"""
        return self.__mul__(scalar)

    def __truediv__(self, scalar: int|float) -> PyGePoint3d:
        """Деление точки на скаляр (Point / Scalar)"""
        if isinstance(scalar, (int,float)):
            if scalar == 0:
                raise ZeroDivisionError("Деление на ноль невозможно.")
            return PyGePoint3d(self.x / scalar, self.y / scalar, self.z / scalar)
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")
    
    def __iadd__(self, vector: PyGeVector3d) -> PyGePoint3d:
        """Оператор += (Point += Vector)"""
        if isinstance(vector, PyGeVector3d):
            self.x += vector.x
            self.y += vector.y
            self.z += vector.z
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def __isub__(self, vector: PyGeVector3d) -> PyGePoint3d:
        """Оператор -= (Point -= Vector)"""
        if isinstance(vector, PyGeVector3d):
            self.x -= vector.x
            self.y -= vector.y
            self.z -= vector.z
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def __imul__(self, scalar: int|float) -> PyGePoint3d:
        """Оператор *= (Point *= Scalar)"""
        if isinstance(scalar, (int, float)):
            self.x *= scalar
            self.y *= scalar
            self.z *= scalar
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")
        
    def __itruediv__(self, scalar: int|float) -> PyGePoint3d:
        """Оператор /= (Point /= Scalar)"""
        if isinstance(scalar, (int,float)):
            if scalar == 0:
                raise ZeroDivisionError("Деление на ноль невозможно.")
            self.x /= scalar
            self.y /= scalar
            self.z /= scalar
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")

    def asVector(self) -> PyGeVector3d:
        """Преобразует точку в вектор, считая ее радиус-вектором от начала координат"""
        return PyGeVector3d(self)
    
    def distanceTo(self, other: PyGePoint3d) -> float:
        """Вычисляет расстояние до другой точки"""
        if isinstance(other, PyGePoint3d):
            dx = self.x - other.x
            dy = self.y - other.y
            dz = self.z - other.z
            return math.sqrt(dx**2 + dy**2 + dz**2)
        raise TypeError(f"Неподдерживаемый тип операнда {type(other)}")
    
    def isEqualTo(self, other_point: PyGePoint3d, tolerance = 1e-9) -> bool:
        """Проверяет равенство точек с учетом допуска"""
        return self.distanceTo(other_point) <= tolerance
    
    def transformBy(self, matrix: PyGeMatrix3d) -> None:
        """Трансформирует текущую точку с помощью матрицы трансформации 4x4."""
        if not isinstance(matrix, PyGeMatrix3d):
            raise TypeError(f"Неподдерживаемый тип операнда {type(matrix)}")
        # Точка в гомогенных координатах: [x, y, z, 1]
        point_vector = np.array([self.x, self.y, self.z, 1.0])
        # Выполняем матричное умножение: M * P
        transformed_point_vector = matrix.matrix @ point_vector
        # Обновляем координаты текущего объекта
        # 0, 1, 2 индексы содержат новые x, y, z
        self.x = transformed_point_vector[0]
        self.y = transformed_point_vector[1]
        self.z = transformed_point_vector[2]
        # return self

    def __str__(self):
        return f"{self.__class__.__name__}(x={self.x:.8f}, y={self.y:.8f}, z={self.z:.8f})"
        
        
        
class PyGePoint2d(vDoubleArray):
    """2d точка"""
    def __init__(self, *args):
        """x=0.0, y=0.0"""
        super().__init__(*args, check_count = 2, set_default = 0.0)

    @property
    def x(self): return self[0]
    @x.setter
    def x(self, value): self[0] = value
    @property
    def y(self): return self[1]
    @y.setter
    def y(self, value): self[1] = value

    def __add__(self, vector: PyGeVector2d) -> PyGePoint2d:
        """Сложение точки с вектором (Point + Vector = Point)"""
        if isinstance(vector, PyGeVector3d):
            return PyGePoint2d(self.x + vector.x,self.y + vector.y)
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")
    
    def __sub__(self, other: PyGePoint2d|PyGeVector2d) -> PyGeVector2d|PyGePoint2d:
        """
        Вычитание:
        1. Точка - Точка = Вектор (Point - Point = Vector)
        2. Точка - Вектор = Точка (Point - Vector = Point)
        """
        if isinstance(other, PyGePoint2d):
            return PyGeVector2d(self.x - other.x, self.y - other.y)
        elif isinstance(other, PyGeVector2d):
            return PyGePoint2d(self.x - other.x, self.y - other.y)
        raise TypeError(f"Неподдерживаемый тип операнда {type(other)}")

    def __mul__(self, scalar: int|float) -> PyGePoint2d:
        """Умножение точки на скаляр (Point * Scalar)"""
        if isinstance(scalar, (int,float)):
            return PyGePoint2d(self.x * scalar, self.y * scalar)
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")

    def __rmul__(self, scalar: int|float) -> PyGePoint2d:
        """Зеркальное умножение на скаляр (Scalar * Point)"""
        return self.__mul__(scalar)

    def __truediv__(self, scalar: int|float) -> PyGePoint2d:
        """Деление точки на скаляр (Point / Scalar)"""
        if isinstance(scalar, (int,float)):
            if scalar == 0:
                raise ZeroDivisionError("Деление на ноль невозможно.")
            return PyGePoint2d(self.x / scalar, self.y / scalar)
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")
    
    def __iadd__(self, vector: PyGeVector2d) -> PyGePoint2d:
        """Оператор += (Point += Vector)"""
        if isinstance(vector, PyGeVector2d):
            self.x += vector.x
            self.y += vector.y
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def __isub__(self, vector: PyGeVector2d) -> PyGePoint2d:
        """Оператор -= (Point -= Vector)"""
        if isinstance(vector, PyGeVector2d):
            self.x -= vector.x
            self.y -= vector.y
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(vector)}")

    def __imul__(self, scalar: int|float) -> PyGePoint2d:
        """Оператор *= (Point *= Scalar)"""
        if isinstance(scalar, (int, float)):
            self.x *= scalar
            self.y *= scalar
            return self
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")
        
    def __itruediv__(self, scalar: int|float) -> PyGePoint2d:
        """Оператор /= (Point /= Scalar)"""
        if isinstance(scalar, (int,float)):
            if scalar == 0:
                raise ZeroDivisionError("Деление на ноль невозможно.")
            self.x /= scalar
            self.y /= scalar
        raise TypeError(f"Неподдерживаемый тип операнда {type(scalar)}")

    def asVector(self) -> PyGeVector2d:
        """Преобразует точку в вектор, считая ее радиус-вектором от начала координат"""
        return PyGeVector2d(self)
    
    def distanceTo(self, other: PyGePoint2d) -> float:
        """Вычисляет расстояние до другой точки"""
        if isinstance(other, PyGeVector2d):
            dx = self.x - other.x
            dy = self.y - other.y
            return math.sqrt(dx**2 + dy**2)
        raise TypeError(f"Неподдерживаемый тип операнда {type(other)}")
    
    def isEqualTo(self, other: PyGePoint2d, tolerance = 1e-9) -> bool:
        """Проверяет равенство точек с учетом допуска"""
        return self.distanceTo(other) <= tolerance
    
    def transformBy(self, matrix: PyGeMatrix2d) -> PyGePoint2d:
        """Трансформирует текущую точку с помощью матрицы трансформации 4x4."""
        if not isinstance(matrix, PyGeMatrix2d):
            raise TypeError("Аргумент должен быть объектом PyGeMatrix3d.")
        # Точка в гомогенных координатах: [x, y, 1]
        point_vector = np.array([self.x, self.y, 1.0])
        # Выполняем матричное умножение: M * P
        # NumPy @ operator выполняет матричное умножение
        transformed_point_vector = matrix.matrix @ point_vector
        # Обновляем координаты текущего объекта
        # 0, 1 индексы содержат новые x, y
        self.x = transformed_point_vector[0]
        self.y = transformed_point_vector[1]
        return self

    def __str__(self):
        return f"{self.__class__.__name__}(x={self.x:.8f}, y={self.y:.8f})"



class PyGePoint3dArray(vDoubleArray):
    def __init__(self, *args):
        arglist = flatten_generic(list(args))
        points3d: list[PyGePoint3d] = []
        for i in range(0, len(arglist), 3):
            chunk = arglist[i:i + 3]
            points3d.append(PyGePoint3d(chunk))
        super().__init__(points3d)

    def __iter__(self):
        for i in range(0, len(self._value_py), 3):
            yield self._value_py[i:i+3]

    def __getitem__(self, index):
        start_index = index * 3
        end_index = start_index + 3
        if end_index > len(self._value_py) and start_index < len(self._value_py):
             return PyGePoint3d(self._value_py[start_index:])
        elif end_index <= len(self._value_py):
            return PyGePoint3d(self._value_py[start_index:end_index])
        else:
            raise IndexError("Out of range")

    def __setitem__(self, index, value):
        if (not isinstance(value, (PyGePoint3d, PyGePoint3dArray))) or (len(value) % 3):
            raise ValueError("Значение должно быть 3-х координатной точкой или массивом точек")
        start_index = index * 3
        end_index = start_index + len(value)
        if end_index <= len(self._value_py):
            self._value_py[start_index:end_index] = value
        else:
            raise IndexError("Вне диапазона")
            
    def __len__(self):
        return int(len(self._value_py) / 3)

    def __add__(self, other):
        if isinstance(other, (PyGePoint3d, PyGePoint3dArray)):
            data_float = self._value_py + other._value_py
        elif isinstance(other, (list,tuple,vDoubleArray)):
            if len(other) % 3:
                raise IndexError("Вне диапазона")
            data_float = self._value_py + flatten_generic(other)
        else:
            raise TypeError(f"Неподдерживаемый тип операнда для сложения: '{other.__class__.__name__}'")
        return PyGePoint3dArray(data_float)
    
    def __repr__(self):
        return f"{self.__class__.__name__}{[PyGePoint3d(v) for v in self]}"
    # def __str__(self):
    #     return f"{tuple([PyGePoint3d(v) for v in self])}"



class PyGePoint2dArray(vDoubleArray):
    def __init__(self, *args):
        arglist = flatten_generic(list(args))
        points2d: list[PyGePoint2d] = []
        for i in range(0, len(arglist), 2):
            chunk = arglist[i:i + 2]
            points2d.append(PyGePoint2d(chunk))
        super().__init__(points2d)

    def __iter__(self):
        for i in range(0, len(self._value_py), 2):
            yield self._value_py[i:i+2]

    def __getitem__(self, index):
        start_index = index * 2
        end_index = start_index + 2
        if end_index > len(self._value_py) and start_index < len(self._value_py):
             return PyGePoint2d(self._value_py[start_index:])
        elif end_index <= len(self._value_py):
            return PyGePoint2d(self._value_py[start_index:end_index])
        else:
            raise IndexError("Вне диапазона")

    def __setitem__(self, index, value):
        if (not isinstance(value, (PyGePoint2d, PyGePoint2dArray))) or (len(value) % 2):
            raise ValueError("Значение должно быть 2-х координатной точкой или массивом точек")
        start_index = index * 2
        end_index = start_index + len(value)
        if end_index <= len(self._value_py):
            self._value_py[start_index:end_index] = value
        else:
            raise IndexError("Вне диапазона")
            
    def __len__(self):
        return int(len(self._value_py) / 2)

    def __add__(self, other):
        if isinstance(other, (PyGePoint2d, PyGePoint2dArray)):
            data_float = self._value_py + other._value_py
        elif isinstance(other, (list,tuple,vDoubleArray)):
            if len(other) % 2:
                raise IndexError("Вне диапазона")
            data_float = self._value_py + flatten_generic(other)
        else:
            raise TypeError(f"Неподдерживаемый тип операнда для сложения: '{other.__class__.__name__}'")
        return PyGePoint2dArray(data_float)
    
    def __repr__(self):
        return f"{self.__class__.__name__}{[PyGePoint2d(v) for v in self]}"



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadSubEntity import *



class AcadSubDMeshEdge(AcadSubEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    CreaseLevel: float = proxy_property(float,'CreaseLevel',AccessMode.ReadWrite)
    CreaseType: AcMeshCreaseType = proxy_property('AcMeshCreaseType','CreaseType',AccessMode.ReadWrite)


from __future__ import annotations
from ..Base import *
from ..AcadObject import *
from ..Proxy import *
from .AcadDocument import AcadDocument



class AcadDocuments(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)

    def Add(self, Name: str = '') -> AcadDocument: 
        return AcadDocument(self._obj.Add(Name))

    def Close(self) -> None: 
        self._obj.Close()

    def Item(self, Index: int) -> AcadDocument: 
        return AcadDocument(self._obj.Item(Index))

    def Open(self, Name: str, ReadOnly: bool = False, Password: Variant = vObjectEmpty) -> AcadDocument: 
        return AcadDocument(self._obj.Open(Name, ReadOnly, Password()))
    
    def __iter__(self):
        for item in self._obj:
            obj = AcadDocument(item)
            yield obj


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadCircle(AcadEntity):
    def __init__(self, obj) -> None: ...
    Area: float
    Center: PyGePoint3d
    Circumference: float
    Diameter: float
    Normal: PyGeVector3d
    Radius: float
    Thickness: float
    def Copy(self) -> AcadCircle: ...
    def Offset(self, Distance: float) -> vObjectArray: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadSurface(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    EdgeExtensionDistances: float = proxy_property(float,'EdgeExtensionDistances',AccessMode.ReadWrite)
    MaintainAssociativity: int = proxy_property(int,'MaintainAssociativity',AccessMode.ReadWrite)
    ShowAssociativity: bool = proxy_property(bool,'ShowAssociativity',AccessMode.ReadWrite)
    SurfaceType: str = proxy_property(str,'SurfaceType',AccessMode.ReadOnly)
    SurfTrimAssociativity: bool = proxy_property(bool,'SurfTrimAssociativity',AccessMode.ReadWrite)
    UIsolineDensity: int = proxy_property(int,'UIsolineDensity',AccessMode.ReadWrite)
    VIsolineDensity: int = proxy_property(int,'VIsolineDensity',AccessMode.ReadWrite)
    WireframeType: AcWireframeType = proxy_property('AcWireframeType','WireframeType',AccessMode.ReadWrite)

    def Copy(self) -> AcadSurface:
        return AcadSurface(self._obj.Copy())


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadSubEntity(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)
    
    Color: AcColor = proxy_property('AcColor','Color',AccessMode.ReadWrite)
    Hyperlinks: AcadHyperlinks = proxy_property('AcadHyperlinks','Hyperlinks',AccessMode.ReadOnly)
    Layer: str = proxy_property(str,'Layer',AccessMode.ReadWrite)
    Linetype: str = proxy_property(str,'Linetype',AccessMode.ReadWrite)
    LinetypeScale: float = proxy_property(float,'LinetypeScale',AccessMode.ReadWrite)
    Lineweight: AcLineWeight = proxy_property('AcLineWeight','Lineweight',AccessMode.ReadWrite)
    ObjectName: str = proxy_property(str,'ObjectName',AccessMode.ReadOnly)
    PlotStyleName: str = proxy_property(str,'PlotStyleName',AccessMode.ReadWrite)



from __future__ import annotations
# from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadAcCmColor(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Blue: int = proxy_property(int,'Blue',AccessMode.ReadOnly)
    BookName: str = proxy_property(str,'BookName',AccessMode.ReadOnly)
    ColorIndex: AcColor = proxy_property('AcColor','ColorIndex',AccessMode.ReadWrite)
    ColorMethod: AcColorMethod = proxy_property('AcColorMethod','ColorMethod',AccessMode.ReadWrite)
    ColorName: str = proxy_property(str,'ColorName',AccessMode.ReadOnly)
    EntityColor: int = proxy_property(int,'EntityColor',AccessMode.ReadWrite)
    Green: int = proxy_property(int,'Green',AccessMode.ReadOnly)
    Red: int = proxy_property(int,'Red',AccessMode.ReadOnly)

    def Delete(self) -> None: 
        self._obj.Delete()
        
    def SetColorBookColor(self, BookName: str, ColorName: str) -> None: 
        self._obj.SetColorBookColor(BookName, ColorName)

    def SetNames(self, ColorName: str, ColorBook: str) -> None: 
        self._obj.SetNames(ColorName, ColorBook)
        
    def SetRGB(self, Red: int, Green: int, Blue: int) -> None: 
        self._obj.SetRGB(Red, Green, Blue)
    


from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferencesOutput(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    AutomaticPlotLog: bool
    ContinuousPlotLog: bool
    DefaultOutputDevice: bool
    DefaultPlotStyleForLayer: str
    DefaultPlotStyleForObjects: str
    DefaultPlotStyleTable: str
    DefaultPlotToFilePath: str
    OLEQuality: AcOleQuality
    PlotLegacy: bool
    PlotPolicy: AcPlotPolicy
    PrinterPaperSizeAlert: bool
    UseLastPlotSettings: bool



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadLayerStateManager(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Mask(self, Name: str) -> AcLayerStateMask|int:
        return self._obj.Mask(Name)

    def Delete(self, Name: str) -> None: 
        self._obj.Delete(Name)
        
    def Export(self, Name: str, FileName: str) -> None: 
        self._obj.Export(Name, FileName)
        
    def Import(self,FileName: str) -> None: 
        self._obj.Import(FileName)

    def Rename(self, OldName: str, NewName: str) -> None: 
        self._obj.Rename(OldName, NewName)
        
    def Restore(self, Name: str) -> None: 
        self._obj.Restore(Name)
        
    def Save(self, Name: str, Mask: AcLayerStateMask|int) -> None: 
        self._obj.Save(Name, Mask)
        
    def SetDatabase(self, Database: AcadDatabase) -> None: 
        self._obj.SetDatabase(Database)
